// Network interface module.
// Purely combinational logic, inserting coordinates of the destination
// and sender tiles before the packet is sent into the network.

`timescale 1ns/1ps
`include "common_defs.v"

module netiface (clk, rst, req_in, rdy_in, data_in, req_out, rdy_out, data_out,
                 req_in_r, rdy_in_r, data_in_r, req_out_r, rdy_out_r, data_out_r);

  // Interconnect link width includes additional bits with
  // (x,y) coordinates of the destination tile.
  parameter link_width_loc = `PACKET_WIDTH;        // local port link width
  parameter link_width_ic  = link_width_loc + 8;   // interconnect link width
  parameter tile_x = 4'b0, tile_y = 4'b0;          // tile coordinates

  parameter apb_i = 0;
  parameter apb_j = 0;
  parameter zbt_i = 0;
  parameter zbt_j = 0;
  parameter acc0_i = 0;
  parameter acc0_j = 0;
  parameter acc1_i = 0;
  parameter acc1_j = 0;
  parameter acc2_i = 0;
  parameter acc2_j = 0;
  parameter acc3_i = 0;
  parameter acc3_j = 0;
  parameter acc4_i = 0;
  parameter acc4_j = 0;
  parameter acc5_i = 0;
  parameter acc5_j = 0;
  parameter acc6_i = 0;
  parameter acc6_j = 0;
  parameter acc7_i = 0;
  parameter acc7_j = 0;
  parameter bram0_i = 0;
  parameter bram0_j = 0;
  parameter bram1_i = 0;
  parameter bram1_j = 0;
  parameter bram2_i = 0;
  parameter bram2_j = 0;
  parameter bram3_i = 0;
  parameter bram3_j = 0;
  parameter bram4_i = 0;
  parameter bram4_j = 0;
  parameter bram5_i = 0;
  parameter bram5_j = 0;
  parameter bram6_i = 0;
  parameter bram6_j = 0;
  parameter bram7_i = 0;
  parameter bram7_j = 0;
  parameter bram8_i = 0;
  parameter bram8_j = 0;
  parameter bram9_i = 0;
  parameter bram9_j = 0;
  parameter brama_i = 0;
  parameter brama_j = 0;
  parameter bramb_i = 0;
  parameter bramb_j = 0;
  parameter bramc_i = 0;
  parameter bramc_j = 0;
  parameter bramd_i = 0;
  parameter bramd_j = 0;
  parameter brame_i = 0;
  parameter brame_j = 0;
  parameter bramf_i = 0;
  parameter bramf_j = 0;

  input  wire clk, rst;

  // interface with local core/mem
  input  wire req_in;
  output wire rdy_in;
  input  wire [link_width_loc-1:0] data_in;

  output wire req_out;
  input  wire rdy_out;
  output wire [link_width_loc-1:0] data_out;
  
  // interface with router
  output wire req_in_r;
  input wire rdy_in_r;
  output wire [link_width_ic-1:0] data_in_r;

  input wire req_out_r;
  output wire rdy_out_r;
  input wire [link_width_ic-1:0] data_out_r;
      
   reg [3:0]                     dest_x;
   reg [3:0]                     dest_y;
   wire [31:0]                   in_addr;
   wire [2:0]                    in_typ;
   wire                          in_reply;
   wire [3:0]                    send_x;
   wire [3:0]                    send_y;
   wire [31:0]                   sys_base = `SYS_BASE;
   wire [31:0]                   bram_base = `BRAM_BASE;
   wire [31:0]                   acc_base = `ACC_BASE;

  assign data_in_r[`DEST_X_END:`DEST_X_BEGIN] = dest_x;
  assign data_in_r[`DEST_Y_END:`DEST_Y_BEGIN] = dest_y;
  assign in_addr = data_in[`ADDR_END:`ADDR_BEGIN];
  assign in_reply = data_in[`REPLY_IDX];
  assign send_x = data_in[`SEND_X_END:`SEND_X_BEGIN];
  assign send_y = data_in[`SEND_Y_END:`SEND_Y_BEGIN];

  // connect req/rdy ports
  assign req_in_r = req_in;
  assign rdy_in = rdy_in_r;
  assign req_out = req_out_r;
  assign rdy_out_r = rdy_out;

  // connect data, add dest and sender coordinates
  
  // in: local -> router
  assign data_in_r[`ADDR_END:`ADDR_BEGIN] = data_in[`ADDR_END:`ADDR_BEGIN];
  assign data_in_r[`REPLY_IDX] = data_in[`REPLY_IDX];
  assign data_in_r[`WR_REQ_IDX] = data_in[`WR_REQ_IDX];
  assign data_in_r[`WR_MASK_END:`WR_MASK_BEGIN] = 
    data_in[`WR_MASK_END:`WR_MASK_BEGIN];
  assign data_in_r[`EXCL_IDX] = data_in[`EXCL_IDX];
  assign data_in_r[`ID_IDX] = data_in[`ID_IDX];
  assign data_in_r[`BURST_IDX] = data_in[`BURST_IDX];
  assign data_in_r[`ERROR_IDX] = data_in[`ERROR_IDX];
  assign data_in_r[`DATA_END:`DATA_BEGIN] = 
     data_in[`DATA_END:`DATA_BEGIN];

  // x-sender: copy for reply, current tile for request
  assign data_in_r[`SEND_X_END:`SEND_X_BEGIN] = 
    (data_in[`REPLY_IDX]) ? data_in[`SEND_X_END:`SEND_X_BEGIN] :  // reply
     tile_x[3:0];                                                 // request

  // y-sender: copy for reply, current tile for request
  assign data_in_r[`SEND_Y_END:`SEND_Y_BEGIN] = 
    (data_in[`REPLY_IDX]) ? data_in[`SEND_Y_END:`SEND_Y_BEGIN] :  // reply
     tile_y[3:0];                                                 // request

  // address decoding
  always @* begin
     dest_x <= zbt_j;
     dest_y <= zbt_i;

     if(in_reply) begin
        dest_x <= send_x;
        dest_y <= send_y;
     end
     else if(in_addr[31:18] == sys_base[31:18]) begin
        if(in_addr[17:16] == sys_base[17:16]) begin
           dest_x <= apb_j;
           dest_y <= apb_i;
        end
        else if(in_addr[17:16] == acc_base[17:16]) begin
           case(in_addr[15:13])
             0: begin
                dest_x <= acc0_j;
                dest_y <= acc0_i;
             end
             1: begin
                dest_x <= acc1_j;
                dest_y <= acc1_i;
             end
             2: begin
                dest_x <= acc2_j;
                dest_y <= acc2_i;
             end
             3: begin
                dest_x <= acc3_j;
                dest_y <= acc3_i;
             end
             4: begin
                dest_x <= acc4_j;
                dest_y <= acc4_i;
             end
             5: begin
                dest_x <= acc5_j;
                dest_y <= acc5_i;
             end
             6: begin
                dest_x <= acc6_j;
                dest_y <= acc6_i;
             end
             7: begin
                dest_x <= acc7_j;
                dest_y <= acc7_i;
             end
           endcase
        end
     end
     else if(in_addr[31:28] == bram_base[31:28]) begin
        case(in_addr[27:24])
          0: begin
             dest_x <= bram0_j;
             dest_y <= bram0_i;
          end
          1: begin
             dest_x <= bram1_j;
             dest_y <= bram1_i;
          end
          2: begin
             dest_x <= bram2_j;
             dest_y <= bram2_i;
          end
          3: begin
             dest_x <= bram3_j;
             dest_y <= bram3_i;
          end
          4: begin
             dest_x <= bram4_j;
             dest_y <= bram4_i;
          end
          5: begin
             dest_x <= bram5_j;
             dest_y <= bram5_i;
          end
          6: begin
             dest_x <= bram6_j;
             dest_y <= bram6_i;
          end
          7: begin
             dest_x <= bram7_j;
             dest_y <= bram7_i;
          end
          8: begin
             dest_x <= bram8_j;
             dest_y <= bram8_i;
          end
          9: begin
             dest_x <= bram9_j;
             dest_y <= bram9_i;
          end
          10: begin
             dest_x <= brama_j;
             dest_y <= brama_i;
          end
          11: begin
             dest_x <= bramb_j;
             dest_y <= bramb_i;
          end
          12: begin
             dest_x <= bramc_j;
             dest_y <= bramc_i;
          end
          13: begin
             dest_x <= bramd_j;
             dest_y <= bramd_i;
          end
          14: begin
             dest_x <= brame_j;
             dest_y <= brame_i;
          end
          15: begin
             dest_x <= bramf_j;
             dest_y <= bramf_i;
          end
        endcase
     end
  end

  // out: router -> local
  // just header+data+sender, no dest
  assign data_out[`PACKET_WIDTH-1:0] = data_out_r[`PACKET_WIDTH-1:0];
  
endmodule
