`timescale 1ns/1ps

// output port is just a fifo in current implementation
module output_port 
  #(
    parameter LINKWIDTH = 188,
    parameter FIFO_ADDR_WIDTH = 1, // log(FIFO_SIZE)
    parameter FIFO_SIZE = 2
    )
  (
   input wire                 clk, 
   input wire                 rst,

   input wire                 write,
   input wire [LINKWIDTH-1:0] data_in,

   input wire                 read,
   output reg [LINKWIDTH-1:0] data_out,

   output reg                 empty,
   output reg                 full
   );

   reg                        fifo_write;
   reg                        fifo_read;
   reg [LINKWIDTH-1:0]        fifo_data_in;
   wire                       fifo_empty;
   wire                       fifo_full;
   wire [LINKWIDTH-1:0]       fifo_data_out;

   // output fifo
   fifo #(
          .WIDTH(LINKWIDTH),
          .ADDR_WIDTH(FIFO_ADDR_WIDTH),
          .SIZE(FIFO_SIZE))
   output_fifo(
               .clk(clk),
               .rst(rst),
               .write(fifo_write),
               .data_in(fifo_data_in),
               .read(fifo_read),
               .data_out(fifo_data_out),
               .empty(fifo_empty),
               .full(fifo_full));

   always @* begin

      fifo_data_in <= data_in;
      fifo_write <= 0;
      fifo_read <= 0;

      data_out <= fifo_data_out;
      empty <= fifo_empty;
      full <= fifo_full;

      if(write & read & fifo_empty) begin
         // bypass fifo, output to next tile directly
         data_out <= data_in;
         empty <= 0;
      end

      if(write & read & !fifo_empty) begin
         // fifo not empty, need to push to fifo
         fifo_write <= 1;
         fifo_read <= 1;
      end

      if(write & !read) begin
         fifo_write <= 1;
      end

      if(!write & read) begin
         fifo_read <= 1;
      end

   end

endmodule

