import uvm_pkg::*;
`include "uvm_macros.svh"
import router_pkg::*;
`include "common_defs.v"

//parameter LINKWIDTH = `IC_WIDTH;  //90
parameter MAX_WAIT = 10;

class router_driver extends uvm_driver#(router_transaction);
	`uvm_component_utils(router_driver);

	virtual router_if vif;

	function new(string name, uvm_component parent);
		super.new(name, parent);
	endfunction: new

	function void build_phase(uvm_phase phase);
		super.build_phase(phase);

		void'(uvm_resource_db#(virtual router_if)::read_by_name(.scope("ifs"), .name("router_if"), .val(vif)));
	endfunction: build_phase

	task run_phase(uvm_phase phase);
		drive();
	endtask: run_phase
   
	// sleep a random number of cycles
   	task sleep(
		input logic clk,
		output logic rdy_out
	);
	logic [31:0] busy;
	integer i;
      	begin
        	busy = ($random & 32'hffff) % MAX_WAIT;
         	if(busy > 0) begin
            		for(i = 0; i < busy; i = i + 1) begin
               			rdy_out = 0;
               			@(posedge clk);
            		end
            	rdy_out = 1;
         	end
      	end
   	endtask

   	virtual task drive();
		router_transaction router_tx;
		
		string data_str; //TEST
		integer c_inport = 0, c_dir = 0, c_test = 0, state = 0, index = 0;
		integer i = 0;
		logic req_out = 1'b0;
		logic [`IC_WIDTH-1:0] data_in[0:4];
		//logic [`IC_WIDTH-1:0] data_out[0:4];

		vif.sig_rst = 1'b1;
		vif.sig_req_in = 5'd0;
        	vif.sig_data_in = 450'd0;
		vif.sig_rdy_out = 5'b11111;
		data_in[0] = 0;
		data_in[1] = 0;
		data_in[2] = 0;
		data_in[3] = 0;
		data_in[4] = 0;
		
		@(posedge vif.sig_clk);
		@(posedge vif.sig_clk);
		vif.sig_rst = 1'b0;
		@(posedge vif.sig_clk);

 		forever begin

			@(posedge vif.sig_clk)
			begin
				
				case(state)
					0: begin
						seq_item_port.get_next_item(router_tx);
						`uvm_info("router_driver", router_tx.sprint(), UVM_LOW);						
						data_str = $sformatf("nr_tests=%2d", router_tx.nr_tests);
						`uvm_info("router_driver", {"TRANSACTION: ",data_str}, UVM_LOW);
						data_str = $sformatf("Array: %b %b %b %b %b %b", router_tx.input_array[0], router_tx.input_array[1], router_tx.input_array[10], router_tx.input_array[11], router_tx.input_array[20], router_tx.input_array[21] );
						`uvm_info("sha1_driver", {"TRANSACTION: ",data_str}, UVM_LOW);
						state = 1;
					end

					1: begin
						vif.sig_req_in <= 5'd0;
						if (c_inport == 5)
							begin
							`uvm_info("router_driver", {"TEST FINISHED"}, UVM_LOW);
							c_inport = 0;
							state = 4;
							i = 0;
							end
						else if (c_dir == 5)
							begin
							c_inport = c_inport + 1;
							c_dir = 0;
							state = 1;
							end
						else if (c_test == router_tx.nr_tests)
							begin
							c_dir = c_dir + 1;
							c_test = 0;	
							state = 1;
							end
						else
							begin
							index = c_dir * router_tx.nr_tests + c_test; 
							data_in[0] = 0;
							data_in[1] = 0;
							data_in[2] = 0;
							data_in[3] = 0;
							data_in[4] = 0;
							case(c_inport)
						        	0: begin
								   data_in[0] = router_tx.input_array[index]; //Input port 0
								   vif.sig_req_in <= 5'b00001;
								   end
								1: begin
								   data_in[1] = router_tx.input_array[index]; //Input port 1
								   vif.sig_req_in <= 5'b00010;
								   end
								2: begin
								   data_in[2] = router_tx.input_array[index]; //Input port 2
								   vif.sig_req_in <= 5'b00100;
								   end	
								3: begin
								   data_in[3] = router_tx.input_array[index]; //Input port 3
								   vif.sig_req_in <= 5'b01000;
								   end
								4: begin
								   data_in[4] = router_tx.input_array[index]; //Input port 4
								   vif.sig_req_in <= 5'b10000;
								   end
							endcase
							vif.sig_data_in = {data_in[4],data_in[3],data_in[2],data_in[1],data_in[0]};
						        c_test = c_test + 1;  
							state = 2;
						end
					end	
					
					2: begin
						if (vif.sig_rdy_in[c_inport] == 1'b1)
							begin
							vif.sig_req_in <= 5'd0;
							state = 3;
							end
						else
							begin
							state = 2;
							end
					end

					3: begin
						case(c_dir)
							0: req_out = vif.sig_req_out[`WEST_DIR]; 	
							1: req_out = vif.sig_req_out[`EAST_DIR]; 
							2: req_out = vif.sig_req_out[`NORTH_DIR]; 
							3: req_out = vif.sig_req_out[`SOUTH_DIR];
 							4: req_out = vif.sig_req_out[`LOCAL_DIR];
						endcase
						if (req_out == 1'b1)
							begin
							state = 1;
							req_out = 1'b0;
							end
						else
							state = 3;
					end

					4: begin //wait for scoreboard
						if (i == 2)
							begin
							i = 0;
							state = 5;
							vif.sig_rst = 1'b1;
							seq_item_port.item_done();
							end
						else
							begin
							i = i + 1;
							state = 4;
							end				
					end

					5: begin
						vif.sig_rst = 1'b0;
						state = 0;	
					end
				endcase
			end
		end 	 

       	endtask: drive
endclass: router_driver