import uvm_pkg::*;
`include "uvm_macros.svh"
import router_pkg::*;
`include "common_defs.v"

//parameter LINKWIDTH = `IC_WIDTH;  //90

class router_monitor_input extends uvm_monitor;
	`uvm_component_utils(router_monitor_input);

	uvm_analysis_port#(router_transaction) mon_ap_minput;

	virtual router_if vif;

	router_transaction router_tx0;
	router_transaction router_tx1;
	router_transaction router_tx2;
	router_transaction router_tx3;
	router_transaction router_tx4;
	
	function new(string name, uvm_component parent);
		super.new(name, parent);
	endfunction: new

	function void build_phase(uvm_phase phase);
		super.build_phase(phase);

		void'(uvm_resource_db#(virtual router_if)::read_by_name(.scope("ifs"), .name("router_if"), .val(vif)));
		mon_ap_minput = new(.name("mon_ap_minput"), .parent(this));
	endfunction: build_phase

	task run_phase(uvm_phase phase);
		integer i = 0, state = 0, nr_tests = 0, c_inports = 0;
		logic c = 0;
		string data_str; //TEST	
		router_tx0 = router_transaction::type_id::create(.name("router_tx0"), .contxt(get_full_name()));
		router_tx1 = router_transaction::type_id::create(.name("router_tx1"), .contxt(get_full_name()));
		router_tx2 = router_transaction::type_id::create(.name("router_tx2"), .contxt(get_full_name()));
		router_tx3 = router_transaction::type_id::create(.name("router_tx3"), .contxt(get_full_name()));
		router_tx4 = router_transaction::type_id::create(.name("router_tx4"), .contxt(get_full_name()));		

		forever begin

			@(posedge vif.sig_clk)
			begin
				case(state)
					0: begin
						if (vif.sig_req_in[1] == 1'b1 && vif.sig_rdy_in[1] == 1'b1)
							begin
							router_tx1.input_array[0] = vif.sig_data_in[`IC_WIDTH*2-1:`IC_WIDTH];
							nr_tests = i;
							i = 1;
							state = 1;
							end
						else if (vif.sig_req_in[0] == 1'b1 && vif.sig_rdy_in[0] == 1'b1)
							begin	 
							router_tx0.input_array[i] = vif.sig_data_in[`IC_WIDTH-1:0];
							//data_str = $sformatf("input_data = %b, i = %d", router_tx0.input_array[i], i);
							//`uvm_info("router_monitor", {"MONITOR INPUT: ",data_str}, UVM_LOW);
							i = i + 1;
							state = 0;
							end	
						else
							state = 0;
						end
					
					1: begin
						if (i == nr_tests)
							begin
							i = 0;
							c_inports = c_inports + 1;
							if (c_inports == 4)
								begin
								c_inports = 0;
								state = 2;
								end
						 	else
								state = 1;
							end
						else if (vif.sig_req_in[1] == 1'b1 && vif.sig_rdy_in[1] == 1'b1)
							begin	 
							router_tx1.input_array[i] = vif.sig_data_in[`IC_WIDTH*2-1:`IC_WIDTH];
							i = i + 1;
							state = 1; 	
							end
						else if (vif.sig_req_in[2] == 1'b1 && vif.sig_rdy_in[2] == 1'b1)
							begin	 
							router_tx2.input_array[i] = vif.sig_data_in[`IC_WIDTH*3-1:`IC_WIDTH*2];
							i = i + 1;
							state = 1; 	
							end
						else if (vif.sig_req_in[3] == 1'b1 && vif.sig_rdy_in[3] == 1'b1)
							begin	 
							router_tx3.input_array[i] = vif.sig_data_in[`IC_WIDTH*4-1:`IC_WIDTH*3];
							i = i + 1;
							state = 1; 	
							end
						else if (vif.sig_req_in[4] == 1'b1 && vif.sig_rdy_in[4] == 1'b1)
							begin	 
							router_tx4.input_array[i] = vif.sig_data_in[`IC_WIDTH*5-1:`IC_WIDTH*4];
							i = i + 1;
							state = 1; 	
							end
						else
							state = 1;
					end
					
					2: begin
						state = 3;
						//Predict the result
						router_tx0.output_array = router_tx0.input_array;
 						//Send the transaction to the analysis port
						mon_ap_minput.write(router_tx0);
					end
					
					3: begin
						state = 4;
						//Predict the result
						router_tx1.output_array = router_tx1.input_array;
 						//Send the transaction to the analysis port
						mon_ap_minput.write(router_tx1);
					end

					4: begin
						state = 5;
						//Predict the result
						router_tx2.output_array = router_tx2.input_array;
 						//Send the transaction to the analysis port
						mon_ap_minput.write(router_tx2);
					end

					5: begin
						state = 6;
						//Predict the result
						router_tx3.output_array = router_tx3.input_array;
 						//Send the transaction to the analysis port
						mon_ap_minput.write(router_tx3);
					end

					6: begin
						state = 0;
						//Predict the result
						router_tx4.output_array = router_tx4.input_array;
 						//Send the transaction to the analysis port
						mon_ap_minput.write(router_tx4);
					end
				endcase
			end
		end
	endtask: run_phase

endclass: router_monitor_input

class router_monitor_output extends uvm_monitor;
	`uvm_component_utils(router_monitor_output)

	uvm_analysis_port#(router_transaction) mon_ap_moutput;

	virtual router_if vif;

	router_transaction router_tx0;
	router_transaction router_tx1;
	router_transaction router_tx2;
	router_transaction router_tx3;
	router_transaction router_tx4;
	
	function new(string name, uvm_component parent);
		super.new(name, parent);
	endfunction: new

	function void build_phase(uvm_phase phase);
		super.build_phase(phase);

		void'(uvm_resource_db#(virtual router_if)::read_by_name(.scope("ifs"), .name("router_if"), .val(vif)));
		mon_ap_moutput= new(.name("mon_ap_moutput"), .parent(this));
	endfunction: build_phase

	task run_phase(uvm_phase phase);
		integer i = 0, state = 0, nr_tests = 0, c_inports = 0;
		string data_str; //TEST

		router_tx0 = router_transaction::type_id::create(.name("router_tx0"), .contxt(get_full_name()));
		router_tx1 = router_transaction::type_id::create(.name("router_tx1"), .contxt(get_full_name()));
		router_tx2 = router_transaction::type_id::create(.name("router_tx2"), .contxt(get_full_name()));
		router_tx3 = router_transaction::type_id::create(.name("router_tx3"), .contxt(get_full_name()));
		router_tx4 = router_transaction::type_id::create(.name("router_tx4"), .contxt(get_full_name()));	

		forever begin
			@(posedge vif.sig_clk)
			begin
				case(state)
					0: begin
						if (vif.sig_req_out[`EAST_DIR] == 1'b1 && vif.sig_rdy_out[`EAST_DIR] == 1'b1)
							begin
							case (c_inports)
								0: begin
								   router_tx0.output_array[i] = vif.sig_data_out[LINKWIDTH*(`EAST_DIR+1)-1:LINKWIDTH*`EAST_DIR];
								   //data_str = $sformatf("output_data = %b", router_tx0.output_array[i]);
								   //`uvm_info("router_monitor", {"MONITOR OUTPUT: ",data_str}, UVM_LOW);	
								   end	
								1: router_tx1.output_array[i] = vif.sig_data_out[LINKWIDTH*(`EAST_DIR+1)-1:LINKWIDTH*`EAST_DIR];
								2: router_tx2.output_array[i] = vif.sig_data_out[LINKWIDTH*(`EAST_DIR+1)-1:LINKWIDTH*`EAST_DIR];
								3: router_tx3.output_array[i] = vif.sig_data_out[LINKWIDTH*(`EAST_DIR+1)-1:LINKWIDTH*`EAST_DIR];
								4: router_tx4.output_array[i] = vif.sig_data_out[LINKWIDTH*(`EAST_DIR+1)-1:LINKWIDTH*`EAST_DIR];
							endcase
							nr_tests = i;
							i = i + 1;
							state = 1;
							end
						else if (vif.sig_req_out[`WEST_DIR] == 1'b1 && vif.sig_rdy_out[`WEST_DIR] == 1'b1)
							begin	 
							case (c_inports)
								0: begin
								   router_tx0.output_array[i] = vif.sig_data_out[LINKWIDTH*(`WEST_DIR+1)-1:LINKWIDTH*`WEST_DIR];
								   //data_str = $sformatf("output_data = %b", router_tx0.output_array[i]);
								   //`uvm_info("router_monitor", {"MONITOR OUTPUT: ",data_str}, UVM_LOW);	
								   end		
								1: router_tx1.output_array[i] = vif.sig_data_out[LINKWIDTH*(`WEST_DIR+1)-1:LINKWIDTH*`WEST_DIR];
								2: router_tx2.output_array[i] = vif.sig_data_out[LINKWIDTH*(`WEST_DIR+1)-1:LINKWIDTH*`WEST_DIR];
								3: router_tx3.output_array[i] = vif.sig_data_out[LINKWIDTH*(`WEST_DIR+1)-1:LINKWIDTH*`WEST_DIR];
								4: router_tx4.output_array[i] = vif.sig_data_out[LINKWIDTH*(`WEST_DIR+1)-1:LINKWIDTH*`WEST_DIR];
							endcase
							i = i + 1;
							state = 0; 	
							end
						else
							state = 0;
						end

					1: begin
						if (i == nr_tests*5)
							begin
							i = 0;
							c_inports = c_inports + 1;
							if (c_inports == 5)
								begin
								c_inports = 0;
								state = 2;
								end
						 	else
								state = 0;
							end
						else if (vif.sig_req_out[`EAST_DIR] == 1'b1 && vif.sig_rdy_out[`EAST_DIR] == 1'b1)
							begin	 
							case (c_inports)
								0: begin
								   router_tx0.output_array[i] = vif.sig_data_out[LINKWIDTH*(`EAST_DIR+1)-1:LINKWIDTH*`EAST_DIR];
								   //data_str = $sformatf("output_data = %b", router_tx0.output_array[i]);
								   //`uvm_info("router_monitor", {"MONITOR OUTPUT: ",data_str}, UVM_LOW);	
								   end
								1: router_tx1.output_array[i] = vif.sig_data_out[LINKWIDTH*(`EAST_DIR+1)-1:LINKWIDTH*`EAST_DIR];
								2: router_tx2.output_array[i] = vif.sig_data_out[LINKWIDTH*(`EAST_DIR+1)-1:LINKWIDTH*`EAST_DIR];   
								3: router_tx3.output_array[i] = vif.sig_data_out[LINKWIDTH*(`EAST_DIR+1)-1:LINKWIDTH*`EAST_DIR];
								4: router_tx4.output_array[i] = vif.sig_data_out[LINKWIDTH*(`EAST_DIR+1)-1:LINKWIDTH*`EAST_DIR];
							endcase
							i = i + 1;
							state = 1; 	
							end
						else if (vif.sig_req_out[`NORTH_DIR] == 1'b1 && vif.sig_rdy_out[`NORTH_DIR] == 1'b1)
							begin	 
							case (c_inports)
								0: begin
								   router_tx0.output_array[i] = vif.sig_data_out[LINKWIDTH*(`NORTH_DIR+1)-1:LINKWIDTH*`NORTH_DIR];
								   //data_str = $sformatf("output_data = %b", router_tx0.output_array[i]);
								   //`uvm_info("router_monitor", {"MONITOR OUTPUT: ",data_str}, UVM_LOW);	
								   end
								1: router_tx1.output_array[i] = vif.sig_data_out[LINKWIDTH*(`NORTH_DIR+1)-1:LINKWIDTH*`NORTH_DIR];
								2: router_tx2.output_array[i] = vif.sig_data_out[LINKWIDTH*(`NORTH_DIR+1)-1:LINKWIDTH*`NORTH_DIR];
								3: router_tx3.output_array[i] = vif.sig_data_out[LINKWIDTH*(`NORTH_DIR+1)-1:LINKWIDTH*`NORTH_DIR];
								4: router_tx4.output_array[i] = vif.sig_data_out[LINKWIDTH*(`NORTH_DIR+1)-1:LINKWIDTH*`NORTH_DIR];
							endcase
							i = i + 1;
							state = 1; 	
							end
						else if (vif.sig_req_out[`SOUTH_DIR] == 1'b1 && vif.sig_rdy_out[`SOUTH_DIR] == 1'b1)
							begin	 
							case (c_inports)
								0: begin
								   router_tx0.output_array[i] = vif.sig_data_out[LINKWIDTH*(`SOUTH_DIR+1)-1:LINKWIDTH*`SOUTH_DIR];
								   //data_str = $sformatf("output_data = %b", router_tx0.output_array[i]);
								   //`uvm_info("router_monitor", {"MONITOR OUTPUT: ",data_str}, UVM_LOW);	
								   end
								1: router_tx1.output_array[i] = vif.sig_data_out[LINKWIDTH*(`SOUTH_DIR+1)-1:LINKWIDTH*`SOUTH_DIR];
								2: router_tx2.output_array[i] = vif.sig_data_out[LINKWIDTH*(`SOUTH_DIR+1)-1:LINKWIDTH*`SOUTH_DIR];
								3: router_tx3.output_array[i] = vif.sig_data_out[LINKWIDTH*(`SOUTH_DIR+1)-1:LINKWIDTH*`SOUTH_DIR];
								4: router_tx4.output_array[i] = vif.sig_data_out[LINKWIDTH*(`SOUTH_DIR+1)-1:LINKWIDTH*`SOUTH_DIR];
							endcase
							i = i + 1;
							state = 1; 	
							end
						else if (vif.sig_req_out[`LOCAL_DIR] == 1'b1 && vif.sig_rdy_out[`LOCAL_DIR] == 1'b1)
							begin	 
							case (c_inports)
								0: begin
								   router_tx0.output_array[i] = vif.sig_data_out[LINKWIDTH*(`LOCAL_DIR+1)-1:LINKWIDTH*`LOCAL_DIR];
								   //data_str = $sformatf("output_data = %b", router_tx0.output_array[i]);
								   //`uvm_info("router_monitor", {"MONITOR OUTPUT: ",data_str}, UVM_LOW);	
								   end
								1: router_tx1.output_array[i] = vif.sig_data_out[LINKWIDTH*(`LOCAL_DIR+1)-1:LINKWIDTH*`LOCAL_DIR];
								2: router_tx2.output_array[i] = vif.sig_data_out[LINKWIDTH*(`LOCAL_DIR+1)-1:LINKWIDTH*`LOCAL_DIR];
								3: router_tx3.output_array[i] = vif.sig_data_out[LINKWIDTH*(`LOCAL_DIR+1)-1:LINKWIDTH*`LOCAL_DIR];
								4: router_tx4.output_array[i] = vif.sig_data_out[LINKWIDTH*(`LOCAL_DIR+1)-1:LINKWIDTH*`LOCAL_DIR];
							endcase
							i = i + 1;
							state = 1; 	
							end
						else
							state = 1;
					end
					
					2: begin
						state = 3;
						//Send the transaction to the analysis port
						mon_ap_moutput.write(router_tx0);
					end
					
					3: begin
						state = 4;
						//Send the transaction to the analysis port
						mon_ap_moutput.write(router_tx1);
					end

					4: begin
						state = 5;
						//Send the transaction to the analysis port
						mon_ap_moutput.write(router_tx2);
					end

					5: begin
						state = 6;
						//Send the transaction to the analysis port
						mon_ap_moutput.write(router_tx3);
					end

					6: begin
						state = 0;
						//Send the transaction to the analysis port
						mon_ap_moutput.write(router_tx4);
					end
				endcase			
			end
		end
	endtask: run_phase
	
endclass: router_monitor_output
