import uvm_pkg::*;
`include "uvm_macros.svh"
import router_pkg::*;
`include "common_defs.v"

class router_s_driver extends uvm_driver#(router_s_transaction);
	`uvm_component_utils(router_s_driver);

	virtual router_if vif;

	function new(string name, uvm_component parent);
		super.new(name, parent);
	endfunction: new

	function void build_phase(uvm_phase phase);
		super.build_phase(phase);

		void'(uvm_resource_db#(virtual router_if)::read_by_name(.scope("ifs"), .name("router_if"), .val(vif)));
	endfunction: build_phase

	task run_phase(uvm_phase phase);
		drive();
	endtask: run_phase
   
	virtual task drive();
		router_s_transaction router_tx;
		
		string data_str; //TEST
		integer state = 0;
		logic req_out = 1'b0;
		logic [`IC_WIDTH-1:0] data_in[0:4];
		
		vif.sig_rst = 1'b1;
		vif.sig_req_in = 5'd0;
        	vif.sig_data_in = 450'd0;
		vif.sig_rdy_out = 5'b11111;
		data_in[0] = 0;
		data_in[1] = 0;
		data_in[2] = 0;
		data_in[3] = 0;
		data_in[4] = 0;
		
		@(posedge vif.sig_clk);
		@(posedge vif.sig_clk);
		vif.sig_rst = 1'b0;
		@(posedge vif.sig_clk);

 		forever begin

			@(posedge vif.sig_clk)
			begin
				
				case(state)
					0: begin
						seq_item_port.get_next_item(router_tx);
						`uvm_info("router_driver", router_tx.sprint(), UVM_LOW);						
						data_str = $sformatf("nr_inport=%2d", router_tx.nr_inport);
						`uvm_info("router_driver", {"TRANSACTION: ",data_str}, UVM_LOW);
						data_str = $sformatf("Input data: %b", router_tx.indata);
						`uvm_info("sha1_driver", {"TRANSACTION: ",data_str}, UVM_LOW);
						state = 1;
					end

					1: begin
						vif.sig_req_in <= 5'd0;
						data_in[0] = 0;
						data_in[1] = 0;
						data_in[2] = 0;
						data_in[3] = 0;
						data_in[4] = 0;
						case(router_tx.nr_inport)
						   	0: begin
							   data_in[0] = router_tx.indata; //Input port 0
							   vif.sig_req_in <= 5'b00001;
							   end
							1: begin
							   data_in[1] = router_tx.indata; //Input port 1
							   vif.sig_req_in <= 5'b00010;
							   end
							2: begin
							   data_in[2] = router_tx.indata; //Input port 2
							   vif.sig_req_in <= 5'b00100;
							   end	
							3: begin
							   data_in[3] = router_tx.indata; //Input port 3
							   vif.sig_req_in <= 5'b01000;
							   end
							4: begin
							   data_in[4] = router_tx.indata; //Input port 4
							   vif.sig_req_in <= 5'b10000;
							   end
						endcase
						vif.sig_data_in = {data_in[4],data_in[3],data_in[2],data_in[1],data_in[0]};
						state = 2;
					end	
					
					2: begin
						if (vif.sig_rdy_in[router_tx.nr_inport] == 1'b1)
							begin
							vif.sig_req_in <= 5'd0;
							state = 3;
							end
						else
							begin
							state = 2;
							end
					end

					3: begin
						req_out = vif.sig_req_out[`WEST_DIR] | vif.sig_req_out[`EAST_DIR] | vif.sig_req_out[`NORTH_DIR] |
							  vif.sig_req_out[`SOUTH_DIR] | vif.sig_req_out[`LOCAL_DIR]; 	
						if (req_out == 1'b1)
							begin
							state = 4;
							req_out = 1'b0;
							end
						else
							state = 3;
					end

					4: begin 
						state = 5;
						vif.sig_rst = 1'b1;
						seq_item_port.item_done();
					end

					5: begin
						vif.sig_rst = 1'b0;
						state = 0;	
					end
				endcase
			end
		end 	 

       	endtask: drive
endclass: router_s_driver