import uvm_pkg::*;
`include "uvm_macros.svh"
import router_pkg::*;
`include "common_defs.v"

class router_s_monitor_input extends uvm_monitor;
	`uvm_component_utils(router_s_monitor_input);

	uvm_analysis_port#(router_s_transaction) mon_ap_minput;

	virtual router_if vif;

	router_s_transaction router_tx;
		
	function new(string name, uvm_component parent);
		super.new(name, parent);
	endfunction: new

	function void build_phase(uvm_phase phase);
		super.build_phase(phase);

		void'(uvm_resource_db#(virtual router_if)::read_by_name(.scope("ifs"), .name("router_if"), .val(vif)));
		mon_ap_minput = new(.name("mon_ap_minput"), .parent(this));
	endfunction: build_phase

	task run_phase(uvm_phase phase);
		integer state = 0;
		//logic [3:0] x_dest, y_dest;
		string data_str; //TEST	
		router_tx = router_s_transaction::type_id::create(.name("router_tx"), .contxt(get_full_name()));

		forever begin

			@(posedge vif.sig_clk)
			begin
				case(state)
					0: begin
						if (vif.sig_req_in[0] == 1'b1 && vif.sig_rdy_in[0] == 1'b1)
							begin
							router_tx.indata = vif.sig_data_in[`IC_WIDTH-1:0]; 
							data_str = $sformatf("input_data = %b", router_tx.indata);
							`uvm_info("router_monitor", {"MONITOR INPUT: ",data_str}, UVM_LOW);
							router_tx.nr_inport = 0;
							state = 1;
							end
						else if (vif.sig_req_in[1] == 1'b1 && vif.sig_rdy_in[1] == 1'b1)
							begin	 
							router_tx.indata = vif.sig_data_in[`IC_WIDTH*2-1:`IC_WIDTH];
							data_str = $sformatf("input_data = %b", router_tx.indata);
							`uvm_info("router_monitor", {"MONITOR INPUT: ",data_str}, UVM_LOW);
							router_tx.nr_inport = 1;
							state = 1;
							end	
						else if (vif.sig_req_in[2] == 1'b1 && vif.sig_rdy_in[2] == 1'b1)
							begin	 
							router_tx.indata = vif.sig_data_in[`IC_WIDTH*3-1:`IC_WIDTH*2];
							data_str = $sformatf("input_data = %b", router_tx.indata);
							`uvm_info("router_monitor", {"MONITOR INPUT: ",data_str}, UVM_LOW);
							router_tx.nr_inport = 2;
							state = 1;
							end	
						else if (vif.sig_req_in[3] == 1'b1 && vif.sig_rdy_in[3] == 1'b1)
							begin	 
							router_tx.indata = vif.sig_data_in[`IC_WIDTH*4-1:`IC_WIDTH*3];
							data_str = $sformatf("input_data = %b", router_tx.indata);
							`uvm_info("router_monitor", {"MONITOR INPUT: ",data_str}, UVM_LOW);
							router_tx.nr_inport = 3;
							state = 1;
							end	
						else if (vif.sig_req_in[4] == 1'b1 && vif.sig_rdy_in[4] == 1'b1)
							begin	 
							router_tx.indata = vif.sig_data_in[`IC_WIDTH*5-1:`IC_WIDTH*4];
							data_str = $sformatf("input_data = %b", router_tx.indata);
							`uvm_info("router_monitor", {"MONITOR INPUT: ",data_str}, UVM_LOW);
							router_tx.nr_inport = 4;
							state = 1;
							end	
						else
							state = 0;
						end
					
					1: begin
						state = 0;
						//Predict the result
						predictor();
						//Send the transaction to the analysis port
						mon_ap_minput.write(router_tx);
					end
				endcase
			end
		end
	endtask: run_phase

	virtual function void predictor();
		logic [3:0] x_dest, y_dest;
		router_tx.outdata = router_tx.indata;   
		//TILE_X = 4'd1; // x-coordinate of current tile
		//TILE_Y = 4'd1; // y-coordinate of current tile
		x_dest = router_tx.indata[`DEST_X_END:`DEST_X_BEGIN];
		y_dest = router_tx.indata[`DEST_Y_END:`DEST_Y_BEGIN];
		if (x_dest == 4'd0)
			router_tx.out_dir = 5'b00100;  //WEST_DIR
		else if (x_dest == 4'd2)
			router_tx.out_dir = 5'b00001;  //EAST_DIR
		else if (y_dest == 4'd0)
			router_tx.out_dir = 5'b00010;  //NORTH_DIR
		else if (y_dest == 4'd2)
			router_tx.out_dir = 5'b01000;  //SOUTH_DIR
		else 
			router_tx.out_dir = 5'b10000;  //LOCAL_DIR	
	endfunction: predictor

endclass: router_s_monitor_input

class router_s_monitor_output extends uvm_monitor;
	`uvm_component_utils(router_s_monitor_output)

	uvm_analysis_port#(router_s_transaction) mon_ap_moutput;

	virtual router_if vif;

	router_s_transaction router_tx;
		
	function new(string name, uvm_component parent);
		super.new(name, parent);
	endfunction: new

	function void build_phase(uvm_phase phase);
		super.build_phase(phase);

		void'(uvm_resource_db#(virtual router_if)::read_by_name(.scope("ifs"), .name("router_if"), .val(vif)));
		mon_ap_moutput= new(.name("mon_ap_moutput"), .parent(this));
	endfunction: build_phase

	task run_phase(uvm_phase phase);
		integer state = 0;
		string data_str; //TEST

		router_tx = router_s_transaction::type_id::create(.name("router_tx"), .contxt(get_full_name()));
		
		forever begin
			@(posedge vif.sig_clk)
			begin
				case(state)
					0: begin
						if (vif.sig_req_out[`EAST_DIR] == 1'b1 && vif.sig_rdy_out[`EAST_DIR] == 1'b1)
							begin
							router_tx.outdata = vif.sig_data_out[LINKWIDTH*(`EAST_DIR+1)-1:LINKWIDTH*`EAST_DIR];
							data_str = $sformatf("output_data = %b", router_tx.outdata);
							`uvm_info("router_monitor", {"MONITOR OUTPUT: ",data_str}, UVM_LOW);	
							router_tx.out_dir = 5'b00001;  //EAST_DIR
							data_str = $sformatf("direction_out = %b, EAST_DIR", router_tx.out_dir);
							`uvm_info("router_monitor", {"MONITOR OUTPUT: ",data_str}, UVM_LOW);	
							state = 1;
							end
						else if (vif.sig_req_out[`WEST_DIR] == 1'b1 && vif.sig_rdy_out[`WEST_DIR] == 1'b1)
							begin	 
							router_tx.outdata = vif.sig_data_out[LINKWIDTH*(`WEST_DIR+1)-1:LINKWIDTH*`WEST_DIR];
							data_str = $sformatf("output_data = %b", router_tx.outdata);
							`uvm_info("router_monitor", {"MONITOR OUTPUT: ",data_str}, UVM_LOW);	
							router_tx.out_dir = 5'b00100;  //WEST_DIR	
							data_str = $sformatf("direction_out = %b, WEST_DIR", router_tx.out_dir);
							`uvm_info("router_monitor", {"MONITOR OUTPUT: ",data_str}, UVM_LOW);	
							state = 1; 	
							end
						else if (vif.sig_req_out[`NORTH_DIR] == 1'b1 && vif.sig_rdy_out[`NORTH_DIR] == 1'b1)
							begin	 
							router_tx.outdata = vif.sig_data_out[LINKWIDTH*(`NORTH_DIR+1)-1:LINKWIDTH*`NORTH_DIR];
							data_str = $sformatf("output_data = %b", router_tx.outdata);
							`uvm_info("router_monitor", {"MONITOR OUTPUT: ",data_str}, UVM_LOW);	
							router_tx.out_dir = 5'b00010;  //NORTH_DIR	
							data_str = $sformatf("direction_out = %b, NORTH_DIR", router_tx.out_dir);
							`uvm_info("router_monitor", {"MONITOR OUTPUT: ",data_str}, UVM_LOW);	
							state = 1; 	
							end
						else if (vif.sig_req_out[`SOUTH_DIR] == 1'b1 && vif.sig_rdy_out[`SOUTH_DIR] == 1'b1)
							begin	 
							router_tx.outdata = vif.sig_data_out[LINKWIDTH*(`SOUTH_DIR+1)-1:LINKWIDTH*`SOUTH_DIR];
							data_str = $sformatf("output_data = %b", router_tx.outdata);
							`uvm_info("router_monitor", {"MONITOR OUTPUT: ",data_str}, UVM_LOW);	
							router_tx.out_dir = 5'b01000;  //SOUTH_DIR	
							data_str = $sformatf("direction_out = %b, SOUTH_DIR", router_tx.out_dir);
							`uvm_info("router_monitor", {"MONITOR OUTPUT: ",data_str}, UVM_LOW);	
							state = 1; 	
							end
						else if (vif.sig_req_out[`LOCAL_DIR] == 1'b1 && vif.sig_rdy_out[`LOCAL_DIR] == 1'b1)
							begin	 
							router_tx.outdata = vif.sig_data_out[LINKWIDTH*(`LOCAL_DIR+1)-1:LINKWIDTH*`LOCAL_DIR];
							data_str = $sformatf("output_data = %b", router_tx.outdata);
							`uvm_info("router_monitor", {"MONITOR OUTPUT: ",data_str}, UVM_LOW);	
							router_tx.out_dir = 5'b10000;  //LOCAL_DIR	
							data_str = $sformatf("direction_out = %b, LOCAL_DIR", router_tx.out_dir);
							`uvm_info("router_monitor", {"MONITOR OUTPUT: ",data_str}, UVM_LOW);	
							state = 1; 	
							end
						else
							state = 0;
						end

					1: begin
						state = 0;
						//Send the transaction to the analysis port
						mon_ap_moutput.write(router_tx);
					end
				endcase			
			end
		end
	endtask: run_phase
	
endclass: router_s_monitor_output
