import uvm_pkg::*;
`include "uvm_macros.svh"
import router_pkg::*;
//`uvm_analysis_imp_decl(_before)
//`uvm_analysis_imp_decl(_after)

class router_scoreboard extends uvm_scoreboard;
	`uvm_component_utils(router_scoreboard);

	uvm_analysis_export #(router_transaction) sb_export_minput;
	uvm_analysis_export #(router_transaction) sb_export_moutput;

	uvm_tlm_analysis_fifo #(router_transaction) minput_fifo;
	uvm_tlm_analysis_fifo #(router_transaction) moutput_fifo;

	router_transaction transaction_minput;
	router_transaction transaction_moutput;

	function new(string name, uvm_component parent);
		super.new(name, parent);

		transaction_minput	= new("transaction_minput");
		transaction_moutput	= new("transaction_moutput");
	endfunction: new

	function void build_phase(uvm_phase phase);
		super.build_phase(phase);

		sb_export_minput	= new("sb_export_minput", this);
		sb_export_moutput	= new("sb_export_moutput", this);

   		minput_fifo		= new("minput_fifo", this);
		moutput_fifo		= new("moutput_fifo", this);
	endfunction: build_phase

	function void connect_phase(uvm_phase phase);
		super.connect_phase(phase);	
		sb_export_minput.connect(minput_fifo.analysis_export);
		sb_export_moutput.connect(moutput_fifo.analysis_export);
	endfunction: connect_phase

	task run();
		forever begin
			minput_fifo.get(transaction_minput);
			moutput_fifo.get(transaction_moutput);
			compare();
		end
	endtask: run

	virtual function void compare();
		if(transaction_minput.output_array == transaction_moutput.output_array) begin
			`uvm_info("compare", {"Test: OK!"}, UVM_LOW);
		end else begin
			`uvm_info("compare", {"Test: Fail!"}, UVM_LOW);
		end
	endfunction: compare
endclass: router_scoreboard