import uvm_pkg::*;
`include "uvm_macros.svh"
import router_pkg::*;
`include "common_defs.v"

parameter TESTS = 10;
parameter LINKWIDTH = `IC_WIDTH;  //90

class router_transaction extends uvm_sequence_item;
	rand int nr_tests;   //TESTS
	rand logic [`IC_WIDTH-1:0] input_array [0:TESTS*5-1]; 
	logic [`IC_WIDTH-1:0] output_array [0:TESTS*5-1];
	

	//constraints
	constraint nr_tests_c {
		nr_tests inside {TESTS};
	}

	task define_directions(); 
		for(int i=0;i<TESTS*5 ;i++) 
		begin 
			if (i<TESTS)
			begin
				input_array[i][`DEST_X_END:`DEST_X_BEGIN] = 4'd0; //WEST
				input_array[i][`DEST_Y_END:`DEST_Y_BEGIN] = 4'd0;
			end
			else if (i>=TESTS && i<TESTS*2)
			begin
				input_array[i][`DEST_X_END:`DEST_X_BEGIN] = 4'd2; //EAST
				input_array[i][`DEST_Y_END:`DEST_Y_BEGIN] = 4'd0;
			end
			else if (i>=TESTS*2 && i<TESTS*3)
			begin
				input_array[i][`DEST_X_END:`DEST_X_BEGIN] = 4'd1; //NORTH
				input_array[i][`DEST_Y_END:`DEST_Y_BEGIN] = 4'd0;
			end
			else if (i>=TESTS*3 && i<TESTS*4)
			begin
				input_array[i][`DEST_X_END:`DEST_X_BEGIN] = 4'd1; //SOUTH
				input_array[i][`DEST_Y_END:`DEST_Y_BEGIN] = 4'd2;
			end
			else 
			begin
				input_array[i][`DEST_X_END:`DEST_X_BEGIN] = 4'd1; //LOCAL
				input_array[i][`DEST_Y_END:`DEST_Y_BEGIN] = 4'd1;
		end
	end
	endtask 

	function new(string name = "");
		super.new(name);
	endfunction: new

	`uvm_object_utils(router_transaction)
	
endclass: router_transaction

class router_sequence extends uvm_sequence#(router_transaction);
	`uvm_object_utils(router_sequence)

	function new(string name = "");
		super.new(name);
	endfunction: new

	task body();
		router_transaction router_tx;
		//router_tx.define_directions(); 

		repeat(10) begin
		router_tx = router_transaction::type_id::create(.name("router_tx"), .contxt(get_full_name()));

		start_item(router_tx);
		assert(router_tx.randomize());
		router_tx.define_directions(); 
		//`uvm_info("router_sequence", router_tx.sprint(), UVM_LOW);
		finish_item(router_tx);
		end
	endtask: body
endclass: router_sequence

typedef uvm_sequencer#(router_transaction) router_sequencer;
