`timescale 1ns/1ps
module QueueSha1_0(input clk, input reset,
    output io_enq_ready,
    input  io_enq_valid,
    input [31:0] io_enq_bits_data,
    input  io_enq_bits_last,
    input  io_deq_ready,
    output io_deq_valid,
    output[31:0] io_deq_bits_data,
    output io_deq_bits_last,
    output[4:0] io_count
);

  wire[4:0] T0;
  wire[3:0] ptr_diff;
  reg [3:0] R1;
  wire[3:0] T19;
  wire[3:0] T2;
  wire[3:0] T3;
  wire do_deq;
  reg [3:0] R4;
  wire[3:0] T20;
  wire[3:0] T5;
  wire[3:0] T6;
  wire do_enq;
  wire T7;
  wire ptr_match;
  reg  maybe_full;
  wire T21;
  wire T8;
  wire T9;
  wire T10;
  wire[32:0] T11;
  reg [32:0] ram [15:0];
  wire[32:0] T12;
  wire[32:0] T13;
  wire[32:0] T14;
  wire[31:0] T15;
  wire T16;
  wire empty;
  wire T17;
  wire T18;
  wire full;

`ifndef SYNTHESIS
// synthesis translate_off
  integer initvar;
  initial begin
    #0.002;
    R1 = {1{$random}};
    R4 = {1{$random}};
    maybe_full = {1{$random}};
    for (initvar = 0; initvar < 16; initvar = initvar+1)
      ram[initvar] = {2{$random}};
  end
// synthesis translate_on
`endif

  assign io_count = T0;
  assign T0 = {T7, ptr_diff};
  assign ptr_diff = R4 - R1;
  assign T19 = reset ? 4'h0 : T2;
  assign T2 = do_deq ? T3 : R1;
  assign T3 = R1 + 4'h1;
  assign do_deq = io_deq_ready & io_deq_valid;
  assign T20 = reset ? 4'h0 : T5;
  assign T5 = do_enq ? T6 : R4;
  assign T6 = R4 + 4'h1;
  assign do_enq = io_enq_ready & io_enq_valid;
  assign T7 = maybe_full & ptr_match;
  assign ptr_match = R4 == R1;
  assign T21 = reset ? 1'h0 : T8;
  assign T8 = T9 ? do_enq : maybe_full;
  assign T9 = do_enq != do_deq;
  assign io_deq_bits_last = T10;
  assign T10 = T11[1'h0];
  assign T11 = ram[R1];
  assign T13 = T14;
  assign T14 = {io_enq_bits_data, io_enq_bits_last};
  assign io_deq_bits_data = T15;
  assign T15 = T11[6'h20:1'h1];
  assign io_deq_valid = T16;
  assign T16 = empty ^ 1'h1;
  assign empty = ptr_match & T17;
  assign T17 = maybe_full ^ 1'h1;
  assign io_enq_ready = T18;
  assign T18 = full ^ 1'h1;
  assign full = ptr_match & maybe_full;

  always @(posedge clk) begin
    if(reset) begin
      R1 <= 4'h0;
    end else if(do_deq) begin
      R1 <= T3;
    end
    if(reset) begin
      R4 <= 4'h0;
    end else if(do_enq) begin
      R4 <= T6;
    end
    if(reset) begin
      maybe_full <= 1'h0;
    end else if(T9) begin
      maybe_full <= do_enq;
    end
    if (do_enq)
      ram[R4] <= T13;
  end
endmodule

module QueueSha1_1(input clk, input reset,
    output io_enq_ready,
    input  io_enq_valid,
    input [31:0] io_enq_bits_data,
    input [31:0] io_enq_bits_addr,
    input [31:0] io_enq_bits_sender,
    input  io_deq_ready,
    output io_deq_valid,
    output[31:0] io_deq_bits_data,
    output[31:0] io_deq_bits_addr,
    output[31:0] io_deq_bits_sender,
    output io_count
);

  wire T13;
  wire[1:0] T0;
  reg  full;
  wire T14;
  wire T1;
  wire do_enq;
  wire T2;
  wire do_deq;
  wire[31:0] T3;
  wire[95:0] T4;
  reg [95:0] ram [0:0];
  wire[95:0] T5;
  wire[95:0] T6;
  wire[95:0] T7;
  wire[63:0] T8;
  wire[31:0] T9;
  wire[31:0] T10;
  wire T11;
  wire empty;
  wire T12;

`ifndef SYNTHESIS
// synthesis translate_off
  integer initvar;
  initial begin
    #0.002;
    full = {1{$random}};
    for (initvar = 0; initvar < 1; initvar = initvar+1)
      ram[initvar] = {3{$random}};
  end
// synthesis translate_on
`endif

  assign io_count = T13;
  assign T13 = T0[1'h0];
  assign T0 = {full, 1'h0};
  assign T14 = reset ? 1'h0 : T1;
  assign T1 = T2 ? do_enq : full;
  assign do_enq = io_enq_ready & io_enq_valid;
  assign T2 = do_enq != do_deq;
  assign do_deq = io_deq_ready & io_deq_valid;
  assign io_deq_bits_sender = T3;
  assign T3 = T4[5'h1f:1'h0];
  assign T4 = ram[1'h0];
  assign T6 = T7;
  assign T7 = {io_enq_bits_data, T8};
  assign T8 = {io_enq_bits_addr, io_enq_bits_sender};
  assign io_deq_bits_addr = T9;
  assign T9 = T4[6'h3f:6'h20];
  assign io_deq_bits_data = T10;
  assign T10 = T4[7'h5f:7'h40];
  assign io_deq_valid = T11;
  assign T11 = empty ^ 1'h1;
  assign empty = full ^ 1'h1;
  assign io_enq_ready = T12;
  assign T12 = full ^ 1'h1;

  always @(posedge clk) begin
    if(reset) begin
      full <= 1'h0;
    end else if(T2) begin
      full <= do_enq;
    end
    if (do_enq)
      ram[1'h0] <= T6;
  end
endmodule

module Sha1(input clk, input reset,
    input  io_req_in,
    output io_rdy_in,
    input [31:0] io_in_addr,
    input  io_in_burst,
    input  io_in_reply,
    input  io_in_write,
    input [3:0] io_in_mask,
    input  io_in_exclusive,
    input [31:0] io_in_data,
    input [8:0] io_in_sender,
    output io_req_out,
    input  io_rdy_out,
    output[31:0] io_out_addr,
    output io_out_burst,
    output io_out_reply,
    output io_out_write,
    output[3:0] io_out_mask,
    output io_out_exclusive,
    output[31:0] io_out_data,
    output[8:0] io_out_sender,
    output io_irq
);

  wire T0;
  wire[31:0] T480;
  wire[31:0] T1;
  wire[31:0] T2;
  wire[31:0] T3;
  wire[31:0] T4;
  wire[31:0] T5;
  wire[31:0] T6;
  wire[31:0] status;
  reg  interrupt;
  wire T481;
  wire T7;
  wire T8;
  wire T9;
  wire T10;
  wire T11;
  wire T12;
  reg [63:0] ml;
  wire[63:0] T482;
  wire[63:0] T13;
  wire[63:0] T14;
  wire[60:0] T15;
  wire T16;
  wire T17;
  wire[2:0] T18;
  wire T19;
  wire T20;
  wire T21;
  wire T22;
  wire T23;
  wire T24;
  wire T25;
  wire T26;
  wire T27;
  wire T28;
  reg [25:0] chunksLeft;
  wire[25:0] T483;
  wire[25:0] T29;
  wire[25:0] T30;
  wire[25:0] T31;
  wire[25:0] T32;
  wire[25:0] T33;
  wire[31:0] T34;
  wire[31:0] T35;
  wire T36;
  wire T37;
  wire[5:0] T38;
  wire[25:0] T39;
  wire T40;
  wire T41;
  wire[25:0] T42;
  wire T43;
  wire gotWord;
  wire T44;
  wire T45;
  wire T46;
  wire T47;
  wire T48;
  wire T49;
  wire T50;
  reg [2:0] memState;
  wire[2:0] T484;
  wire[2:0] T51;
  wire[2:0] T52;
  wire[2:0] T53;
  wire[2:0] T54;
  wire[2:0] T55;
  wire[2:0] T56;
  wire[2:0] T57;
  wire[2:0] T58;
  wire[2:0] T59;
  wire T60;
  wire T61;
  reg [3:0] wordInChunk;
  wire[3:0] T485;
  wire[3:0] T62;
  wire[3:0] T63;
  wire[3:0] T64;
  wire[3:0] T65;
  wire[3:0] T66;
  wire T67;
  wire T68;
  wire T69;
  wire T70;
  wire T71;
  reg [1:0] byteTrunc;
  wire[1:0] T486;
  wire[1:0] T72;
  wire[1:0] T73;
  wire T74;
  wire T75;
  wire T76;
  wire T77;
  wire T78;
  wire T79;
  wire memfill;
  wire T80;
  wire T81;
  wire T82;
  wire T83;
  reg [2:0] hashState;
  wire[2:0] T487;
  wire[2:0] T84;
  wire[2:0] T85;
  wire[2:0] T86;
  wire[2:0] T87;
  wire[2:0] T88;
  wire[2:0] T89;
  wire[2:0] T90;
  wire[2:0] T91;
  wire T92;
  wire T93;
  reg  hashing;
  wire T488;
  wire T94;
  wire T95;
  wire T96;
  wire counterZero;
  reg [4:0] hashCounter;
  wire[4:0] T489;
  wire[4:0] T97;
  wire[4:0] T98;
  wire[4:0] T99;
  wire[4:0] T100;
  wire[4:0] T101;
  wire[4:0] T102;
  wire[4:0] T103;
  wire[4:0] T104;
  wire T105;
  wire stall;
  wire T106;
  wire T107;
  wire T108;
  wire T109;
  wire T110;
  wire T111;
  wire T112;
  wire T113;
  wire T114;
  wire T115;
  wire T116;
  wire T117;
  wire T118;
  wire T119;
  wire T120;
  wire T121;
  wire T122;
  wire T123;
  wire T124;
  wire T125;
  wire T126;
  wire T127;
  wire[30:0] T128;
  wire[29:0] T129;
  reg  prefetching;
  wire T490;
  wire T130;
  wire T131;
  wire T132;
  wire T133;
  reg [29:0] pfWordsLeft;
  wire[29:0] T491;
  wire[29:0] T134;
  wire[29:0] T135;
  wire[29:0] T136;
  wire[29:0] T137;
  wire[29:0] T138;
  wire[29:0] T139;
  wire[29:0] T140;
  wire[29:0] T141;
  wire[29:0] T142;
  wire T143;
  wire T144;
  wire[3:0] T145;
  reg [1:0] byteShift;
  wire[1:0] T492;
  wire[1:0] T146;
  wire[1:0] T147;
  wire T148;
  wire T149;
  wire T150;
  wire T151;
  wire[3:0] T152;
  wire T153;
  wire T154;
  wire[3:0] T155;
  wire[29:0] T156;
  wire[29:0] T157;
  wire T158;
  wire T159;
  wire[3:0] T160;
  wire[29:0] T161;
  wire T162;
  wire T163;
  wire T164;
  reg [5:0] pfCounter;
  wire[5:0] T493;
  wire[5:0] T165;
  wire[5:0] T166;
  wire[5:0] T167;
  wire[5:0] T168;
  wire T169;
  wire T170;
  wire T171;
  wire T172;
  wire[5:0] T173;
  wire pfNow;
  wire[5:0] T174;
  wire T175;
  wire T176;
  wire T177;
  wire T178;
  wire[28:0] T179;
  reg  error;
  wire T494;
  wire T180;
  wire T181;
  wire T182;
  wire T183;
  wire T184;
  wire[2:0] T185;
  wire T186;
  wire T187;
  wire T188;
  wire[31:0] T189;
  wire[7:0] T190;
  reg [31:0] h0;
  wire[31:0] T495;
  wire[31:0] T191;
  wire[31:0] T192;
  wire[31:0] T193;
  reg [31:0] a;
  wire[31:0] T496;
  wire[31:0] T194;
  wire[31:0] T195;
  wire[31:0] T196;
  wire[31:0] T197;
  reg [31:0] w0;
  wire[31:0] T497;
  wire[32:0] T498;
  wire[32:0] T198;
  wire[32:0] T199;
  wire[32:0] T499;
  wire[31:0] T200;
  wire[31:0] T201;
  wire[31:0] T202;
  wire[31:0] T203;
  wire[31:0] T204;
  wire[31:0] T205;
  wire[31:0] T206;
  wire[31:0] T207;
  wire[31:0] T208;
  wire[7:0] T209;
  wire[23:0] T210;
  wire[7:0] T211;
  wire[15:0] T212;
  wire[7:0] T213;
  wire[7:0] T214;
  wire[31:0] T215;
  wire[7:0] T216;
  wire T217;
  wire T218;
  wire[31:0] T219;
  wire[15:0] T220;
  wire[7:0] T221;
  wire[7:0] T222;
  wire T223;
  wire T224;
  wire[31:0] T225;
  wire[23:0] T226;
  wire[7:0] T227;
  wire[15:0] T228;
  wire[7:0] T229;
  wire[7:0] T230;
  wire T231;
  wire T232;
  wire[31:0] T233;
  wire[31:0] T234;
  wire[32:0] T235;
  wire[32:0] T500;
  wire[31:0] T236;
  wire[30:0] T237;
  wire[30:0] T238;
  wire[31:0] T239;
  reg [31:0] w15;
  wire[31:0] T501;
  wire[31:0] T240;
  reg [31:0] w14;
  wire[31:0] T502;
  wire[31:0] T241;
  reg [31:0] w13;
  wire[31:0] T503;
  wire[31:0] T242;
  reg [31:0] w12;
  wire[31:0] T504;
  wire[31:0] T243;
  reg [31:0] w11;
  wire[31:0] T505;
  wire[31:0] T244;
  reg [31:0] w10;
  wire[31:0] T506;
  wire[31:0] T245;
  reg [31:0] w9;
  wire[31:0] T507;
  wire[31:0] T246;
  reg [31:0] w8;
  wire[31:0] T508;
  wire[31:0] T247;
  reg [31:0] w7;
  wire[31:0] T509;
  wire[31:0] T248;
  reg [31:0] w6;
  wire[31:0] T510;
  wire[31:0] T249;
  reg [31:0] w5;
  wire[31:0] T511;
  wire[31:0] T250;
  reg [31:0] w4;
  wire[31:0] T512;
  wire[31:0] T251;
  reg [31:0] w3;
  wire[31:0] T513;
  wire[31:0] T252;
  reg [31:0] w2;
  wire[31:0] T514;
  wire[31:0] T253;
  reg [31:0] w1;
  wire[31:0] T515;
  wire[31:0] T254;
  wire[31:0] T255;
  wire[31:0] T256;
  wire[32:0] T257;
  wire[32:0] T258;
  wire[32:0] T259;
  wire[32:0] T516;
  wire[32:0] T260;
  wire[32:0] T517;
  wire[1:0] T261;
  wire[1:0] T262;
  wire[1:0] T518;
  wire T263;
  wire T264;
  wire T265;
  wire[30:0] T519;
  wire T520;
  wire[32:0] T266;
  wire[32:0] T521;
  wire[1:0] T267;
  wire[30:0] T522;
  wire T523;
  wire T268;
  wire[31:0] T269;
  wire[31:0] k;
  wire[31:0] T270;
  wire[31:0] T271;
  wire[31:0] T272;
  wire[31:0] T273;
  wire[31:0] T274;
  wire[31:0] T275;
  reg [31:0] e;
  wire[31:0] T524;
  wire[31:0] T276;
  wire[31:0] T277;
  wire[31:0] T278;
  reg [31:0] d;
  wire[31:0] T525;
  wire[31:0] T279;
  wire[31:0] T280;
  wire[31:0] T281;
  reg [31:0] c;
  wire[31:0] T526;
  wire[31:0] T282;
  wire[31:0] T283;
  wire[31:0] T284;
  wire[31:0] T285;
  wire[29:0] T286;
  reg [31:0] b;
  wire[31:0] T527;
  wire[31:0] T287;
  wire[31:0] T288;
  wire[31:0] T289;
  wire[31:0] T290;
  reg [31:0] h1;
  wire[31:0] T528;
  wire[31:0] T291;
  wire[31:0] T292;
  wire[1:0] T293;
  wire[31:0] T294;
  reg [31:0] h2;
  wire[31:0] T529;
  wire[31:0] T295;
  wire[31:0] T296;
  wire[31:0] T297;
  reg [31:0] h3;
  wire[31:0] T530;
  wire[31:0] T298;
  wire[31:0] T299;
  wire[31:0] T300;
  reg [31:0] h4;
  wire[31:0] T531;
  wire[31:0] T301;
  wire[31:0] T302;
  wire[31:0] T303;
  wire[31:0] f;
  wire[31:0] T304;
  wire[31:0] T305;
  wire[31:0] T306;
  wire[31:0] T307;
  wire[31:0] T308;
  wire[31:0] T309;
  wire[31:0] T310;
  wire[31:0] T311;
  wire[31:0] T312;
  wire[31:0] T313;
  wire[31:0] T314;
  wire[31:0] T315;
  wire[31:0] T316;
  wire[31:0] T317;
  wire[31:0] T318;
  wire[31:0] T319;
  wire[31:0] T320;
  wire[31:0] T321;
  wire[31:0] T322;
  wire[31:0] T323;
  wire[31:0] T324;
  wire[31:0] T325;
  wire[31:0] T326;
  wire[4:0] T327;
  wire[26:0] T328;
  wire[23:0] T329;
  wire[7:0] T330;
  wire[15:0] T331;
  wire[7:0] T332;
  wire[7:0] T333;
  wire T334;
  wire T335;
  wire[31:0] T336;
  wire[7:0] T337;
  wire[23:0] T338;
  wire[7:0] T339;
  wire[15:0] T340;
  wire[7:0] T341;
  wire[7:0] T342;
  wire T343;
  wire T344;
  wire[31:0] T345;
  wire[7:0] T346;
  wire[23:0] T347;
  wire[7:0] T348;
  wire[15:0] T349;
  wire[7:0] T350;
  wire[7:0] T351;
  wire T352;
  wire T353;
  wire[31:0] T354;
  wire[7:0] T355;
  wire[23:0] T356;
  wire[7:0] T357;
  wire[15:0] T358;
  wire[7:0] T359;
  wire[7:0] T360;
  wire T361;
  wire T362;
  wire[31:0] T363;
  wire[7:0] T364;
  wire[23:0] T365;
  wire[7:0] T366;
  wire[15:0] T367;
  wire[7:0] T368;
  wire[7:0] T369;
  wire T370;
  wire T371;
  wire T372;
  wire T373;
  wire T374;
  wire T375;
  wire T376;
  wire T377;
  reg  lastWord;
  wire T532;
  wire T378;
  wire T379;
  wire T380;
  wire T381;
  wire T382;
  wire T383;
  wire T384;
  wire T385;
  wire T386;
  wire T387;
  wire T388;
  wire T389;
  wire T390;
  wire T391;
  reg [29:0] replyWordsLeft;
  wire[29:0] T533;
  wire[29:0] T392;
  wire[29:0] T393;
  wire[29:0] T394;
  wire[29:0] T395;
  wire T396;
  wire T397;
  wire T398;
  wire T399;
  wire T400;
  wire T401;
  wire T402;
  wire T403;
  wire T404;
  wire T405;
  wire T406;
  wire T407;
  wire T408;
  wire T409;
  wire T410;
  wire T411;
  wire T412;
  wire T413;
  wire T414;
  wire T415;
  wire T416;
  wire T417;
  wire T418;
  wire T419;
  wire T420;
  wire T421;
  wire[31:0] T534;
  wire[47:0] T422;
  wire[47:0] T423;
  wire[47:0] T424;
  wire[47:0] T425;
  wire[47:0] T426;
  wire[47:0] T427;
  wire[47:0] T428;
  wire[47:0] T429;
  wire[23:0] T430;
  reg [31:0] storedData;
  wire[31:0] T535;
  wire[31:0] T431;
  wire T432;
  wire T433;
  wire[47:0] T536;
  wire[31:0] T434;
  wire[15:0] T435;
  wire T436;
  wire T437;
  wire[47:0] T537;
  wire[15:0] T438;
  wire[7:0] T439;
  wire T440;
  wire T441;
  wire[47:0] T538;
  wire T442;
  wire T443;
  wire[47:0] T539;
  wire[31:0] T444;
  wire[23:0] T445;
  wire[7:0] T446;
  wire T447;
  wire T448;
  wire T449;
  wire T450;
  reg  storedDataValid;
  wire T540;
  wire T451;
  wire T452;
  wire T453;
  wire[47:0] T541;
  wire[31:0] T454;
  wire[15:0] T455;
  wire[15:0] T456;
  wire T457;
  wire T458;
  wire[47:0] T542;
  wire[31:0] T459;
  wire[7:0] T460;
  wire[23:0] T461;
  wire T462;
  wire T463;
  wire T464;
  wire T465;
  wire T466;
  wire T467;
  wire[8:0] T543;
  wire[31:0] T468;
  wire[31:0] T469;
  wire[31:0] T470;
  wire[31:0] T471;
  wire[31:0] T472;
  reg [31:0] addr;
  wire[31:0] T544;
  wire[31:0] T473;
  wire[31:0] T474;
  wire[31:0] T475;
  wire[29:0] T476;
  wire[31:0] T477;
  wire T478;
  wire T479;
  wire queue_io_enq_ready;
  wire queue_io_deq_valid;
  wire[31:0] queue_io_deq_bits_data;
  wire queue_io_deq_bits_last;
  wire replyQueueSha1_io_enq_ready;
  wire replyQueueSha1_io_deq_valid;
  wire[31:0] replyQueueSha1_io_deq_bits_data;
  wire[31:0] replyQueueSha1_io_deq_bits_addr;
  wire[31:0] replyQueueSha1_io_deq_bits_sender;

`ifndef SYNTHESIS
// synthesis translate_off
  integer initvar;
  initial begin
    #0.002;
    interrupt = {1{$random}};
    ml = {2{$random}};
    chunksLeft = {1{$random}};
    memState = {1{$random}};
    wordInChunk = {1{$random}};
    byteTrunc = {1{$random}};
    hashState = {1{$random}};
    hashing = {1{$random}};
    hashCounter = {1{$random}};
    prefetching = {1{$random}};
    pfWordsLeft = {1{$random}};
    byteShift = {1{$random}};
    pfCounter = {1{$random}};
    error = {1{$random}};
    h0 = {1{$random}};
    a = {1{$random}};
    w0 = {1{$random}};
    w15 = {1{$random}};
    w14 = {1{$random}};
    w13 = {1{$random}};
    w12 = {1{$random}};
    w11 = {1{$random}};
    w10 = {1{$random}};
    w9 = {1{$random}};
    w8 = {1{$random}};
    w7 = {1{$random}};
    w6 = {1{$random}};
    w5 = {1{$random}};
    w4 = {1{$random}};
    w3 = {1{$random}};
    w2 = {1{$random}};
    w1 = {1{$random}};
    e = {1{$random}};
    d = {1{$random}};
    c = {1{$random}};
    b = {1{$random}};
    h1 = {1{$random}};
    h2 = {1{$random}};
    h3 = {1{$random}};
    h4 = {1{$random}};
    lastWord = {1{$random}};
    replyWordsLeft = {1{$random}};
    storedData = {1{$random}};
    storedDataValid = {1{$random}};
    addr = {1{$random}};
  end
// synthesis translate_on
`endif

  assign T0 = replyQueueSha1_io_deq_valid & io_rdy_out;
  assign T480 = {23'h0, io_in_sender};
  assign T1 = T370 ? T363 : T2;
  assign T2 = T361 ? T354 : T3;
  assign T3 = T352 ? T345 : T4;
  assign T4 = T343 ? T336 : T5;
  assign T5 = T334 ? T189 : T6;
  assign T6 = T183 ? status : 32'h0;
  assign status = {T128, interrupt};
  assign T481 = reset ? 1'h0 : T7;
  assign T7 = T27 ? 1'h1 : T8;
  assign T8 = T183 ? 1'h0 : T9;
  assign T9 = T25 ? 1'h0 : T10;
  assign T10 = T11 ? 1'h1 : interrupt;
  assign T11 = T23 & T12;
  assign T12 = ml == 64'h0;
  assign T482 = reset ? 64'h0 : T13;
  assign T13 = T16 ? T14 : ml;
  assign T14 = {T15, 3'h0};
  assign T15 = {29'h0, io_in_data};
  assign T16 = T19 & T17;
  assign T17 = 3'h1 == T18;
  assign T18 = io_in_addr[3'h4:2'h2];
  assign T19 = T22 & T20;
  assign T20 = T21 & io_in_write;
  assign T21 = io_in_reply ^ 1'h1;
  assign T22 = io_req_in & io_rdy_in;
  assign T23 = T19 & T24;
  assign T24 = 3'h2 == T18;
  assign T25 = T23 & T26;
  assign T26 = T12 ^ 1'h1;
  assign T27 = T127 & T28;
  assign T28 = chunksLeft == 26'h1;
  assign T483 = reset ? 26'h0 : T29;
  assign T29 = T43 ? T42 : T30;
  assign T30 = T40 ? T39 : T31;
  assign T31 = T36 ? T32 : chunksLeft;
  assign T32 = T33 + 26'h1;
  assign T33 = T34[5'h1f:3'h6];
  assign T34 = T35 + 32'h9;
  assign T35 = ml[6'h22:2'h3];
  assign T36 = T25 & T37;
  assign T37 = T38 != 6'h0;
  assign T38 = T34[3'h5:1'h0];
  assign T39 = T34[5'h1f:3'h6];
  assign T40 = T25 & T41;
  assign T41 = T37 ^ 1'h1;
  assign T42 = chunksLeft - 26'h1;
  assign T43 = T125 & gotWord;
  assign gotWord = T44;
  assign T44 = T123 ? 1'h1 : T45;
  assign T45 = T121 ? 1'h1 : T46;
  assign T46 = T119 ? 1'h1 : T47;
  assign T47 = T117 ? 1'h1 : T48;
  assign T48 = T49 & queue_io_deq_valid;
  assign T49 = memfill & T50;
  assign T50 = 3'h0 == memState;
  assign T484 = reset ? 3'h0 : T51;
  assign T51 = T123 ? 3'h0 : T52;
  assign T52 = T121 ? 3'h4 : T53;
  assign T53 = T78 ? 3'h3 : T54;
  assign T54 = T76 ? 3'h2 : T55;
  assign T55 = T74 ? 3'h3 : T56;
  assign T56 = T70 ? 3'h1 : T57;
  assign T57 = T68 ? 3'h2 : T58;
  assign T58 = T60 ? 3'h3 : T59;
  assign T59 = T25 ? 3'h0 : memState;
  assign T60 = T67 & T61;
  assign T61 = wordInChunk == 4'hd;
  assign T485 = reset ? 4'h0 : T62;
  assign T62 = T119 ? T66 : T63;
  assign T63 = T117 ? T66 : T64;
  assign T64 = T48 ? T66 : T65;
  assign T65 = T25 ? 4'h0 : wordInChunk;
  assign T66 = wordInChunk + 4'h1;
  assign T67 = T48 & queue_io_deq_bits_last;
  assign T68 = T67 & T69;
  assign T69 = T61 ^ 1'h1;
  assign T70 = T67 & T71;
  assign T71 = 2'h0 == byteTrunc;
  assign T486 = reset ? 2'h0 : T72;
  assign T72 = T16 ? T73 : byteTrunc;
  assign T73 = io_in_data[1'h1:1'h0];
  assign T74 = T117 & T75;
  assign T75 = wordInChunk == 4'hd;
  assign T76 = T117 & T77;
  assign T77 = T75 ^ 1'h1;
  assign T78 = T119 & T79;
  assign T79 = wordInChunk == 4'hd;
  assign memfill = T80;
  assign T80 = T125 ? 1'h1 : T81;
  assign T81 = T116 ? 1'h0 : T82;
  assign T82 = T115 ? 1'h1 : T83;
  assign T83 = 3'h0 == hashState;
  assign T487 = reset ? 3'h0 : T84;
  assign T84 = T43 ? 3'h1 : T85;
  assign T85 = T27 ? 3'h0 : T86;
  assign T86 = T113 ? 3'h6 : T87;
  assign T87 = T111 ? 3'h5 : T88;
  assign T88 = T109 ? 3'h4 : T89;
  assign T89 = T96 ? 3'h3 : T90;
  assign T90 = T116 ? 3'h2 : T91;
  assign T91 = T92 ? 3'h1 : hashState;
  assign T92 = T93 & gotWord;
  assign T93 = T83 & hashing;
  assign T488 = reset ? 1'h0 : T94;
  assign T94 = T27 ? 1'h0 : T95;
  assign T95 = T25 ? 1'h1 : hashing;
  assign T96 = T108 & counterZero;
  assign counterZero = hashCounter == 5'h0;
  assign T489 = reset ? 5'h0 : T97;
  assign T97 = T125 ? 5'hf : T98;
  assign T98 = T111 ? 5'h13 : T99;
  assign T99 = T109 ? 5'h13 : T100;
  assign T100 = T96 ? 5'h13 : T101;
  assign T101 = T116 ? 5'h3 : T102;
  assign T102 = T93 ? 5'hf : T103;
  assign T103 = T105 ? T104 : hashCounter;
  assign T104 = hashCounter - 5'h1;
  assign T105 = stall ^ 1'h1;
  assign stall = T106;
  assign T106 = T49 & T107;
  assign T107 = queue_io_deq_valid ^ 1'h1;
  assign T108 = 3'h2 == hashState;
  assign T109 = T110 & counterZero;
  assign T110 = 3'h3 == hashState;
  assign T111 = T112 & counterZero;
  assign T112 = 3'h4 == hashState;
  assign T113 = T114 & counterZero;
  assign T114 = 3'h5 == hashState;
  assign T115 = 3'h1 == hashState;
  assign T116 = T115 & counterZero;
  assign T117 = memfill & T118;
  assign T118 = 3'h1 == memState;
  assign T119 = memfill & T120;
  assign T120 = 3'h2 == memState;
  assign T121 = memfill & T122;
  assign T122 = 3'h3 == memState;
  assign T123 = memfill & T124;
  assign T124 = 3'h4 == memState;
  assign T125 = T127 & T126;
  assign T126 = T28 ^ 1'h1;
  assign T127 = 3'h6 == hashState;
  assign T128 = {T129, hashing};
  assign T129 = {T179, prefetching};
  assign T490 = reset ? 1'h0 : T130;
  assign T130 = T132 ? 1'h0 : T131;
  assign T131 = T25 ? 1'h1 : prefetching;
  assign T132 = T162 & T133;
  assign T133 = pfWordsLeft == 30'h1;
  assign T491 = reset ? 30'h0 : T134;
  assign T134 = T162 ? T161 : T135;
  assign T135 = T158 ? T156 : T136;
  assign T136 = T153 ? T156 : T137;
  assign T137 = T150 ? T156 : T138;
  assign T138 = T143 ? T141 : T139;
  assign T139 = T25 ? T140 : pfWordsLeft;
  assign T140 = ml[6'h22:3'h5];
  assign T141 = T142 + 30'h1;
  assign T142 = ml[6'h22:3'h5];
  assign T143 = T25 & T144;
  assign T144 = T145 != 4'h0;
  assign T145 = {byteShift, byteTrunc};
  assign T492 = reset ? 2'h0 : T146;
  assign T146 = T148 ? T147 : byteShift;
  assign T147 = io_in_data[1'h1:1'h0];
  assign T148 = T19 & T149;
  assign T149 = 3'h0 == T18;
  assign T150 = T25 & T151;
  assign T151 = T152 == 4'hb;
  assign T152 = {byteShift, byteTrunc};
  assign T153 = T25 & T154;
  assign T154 = T155 == 4'he;
  assign T155 = {byteShift, byteTrunc};
  assign T156 = T157 + 30'h2;
  assign T157 = ml[6'h22:3'h5];
  assign T158 = T25 & T159;
  assign T159 = T160 == 4'hf;
  assign T160 = {byteShift, byteTrunc};
  assign T161 = pfWordsLeft - 30'h1;
  assign T162 = T163 & io_rdy_out;
  assign T163 = T175 & T164;
  assign T164 = pfCounter != 6'h0;
  assign T493 = reset ? 6'h0 : T165;
  assign T165 = T162 ? T174 : T166;
  assign T166 = pfNow ? T173 : T167;
  assign T167 = T171 ? 6'h21 : T168;
  assign T168 = T169 ? 6'h20 : pfCounter;
  assign T169 = T25 & T170;
  assign T170 = byteShift == 2'h0;
  assign T171 = T25 & T172;
  assign T172 = T170 ^ 1'h1;
  assign T173 = pfCounter + 6'h10;
  assign pfNow = T43;
  assign T174 = pfCounter - 6'h1;
  assign T175 = T178 & T176;
  assign T176 = prefetching & T177;
  assign T177 = ~ pfNow;
  assign T178 = replyQueueSha1_io_deq_valid ^ 1'h1;
  assign T179 = {28'h0, error};
  assign T494 = reset ? 1'h0 : T180;
  assign T180 = T183 ? 1'h0 : T181;
  assign T181 = T25 ? 1'h0 : T182;
  assign T182 = T11 ? 1'h1 : error;
  assign T183 = T186 & T184;
  assign T184 = 3'h2 == T185;
  assign T185 = io_in_addr[3'h4:2'h2];
  assign T186 = T22 & T187;
  assign T187 = T188 ^ 1'h1;
  assign T188 = io_in_reply | io_in_write;
  assign T189 = {T329, T190};
  assign T190 = h0[5'h1f:5'h18];
  assign T495 = reset ? 32'h0 : T191;
  assign T191 = T127 ? T193 : T192;
  assign T192 = T93 ? 32'h67452301 : h0;
  assign T193 = h0 + a;
  assign T496 = reset ? 32'h0 : T194;
  assign T194 = T127 ? T193 : T195;
  assign T195 = T93 ? 32'h67452301 : T196;
  assign T196 = T105 ? T197 : a;
  assign T197 = T269 + w0;
  assign T497 = T498[5'h1f:1'h0];
  assign T498 = reset ? 33'h0 : T198;
  assign T198 = T268 ? T260 : T199;
  assign T199 = T268 ? T235 : T499;
  assign T499 = {1'h0, T200};
  assign T200 = T123 ? T234 : T201;
  assign T201 = T121 ? T233 : T202;
  assign T202 = T119 ? 32'h0 : T203;
  assign T203 = T117 ? 32'h80000000 : T204;
  assign T204 = T231 ? T225 : T205;
  assign T205 = T223 ? T219 : T206;
  assign T206 = T217 ? T215 : T207;
  assign T207 = T48 ? T208 : w0;
  assign T208 = {T210, T209};
  assign T209 = queue_io_deq_bits_data[5'h1f:5'h18];
  assign T210 = {T212, T211};
  assign T211 = queue_io_deq_bits_data[5'h17:5'h10];
  assign T212 = {T214, T213};
  assign T213 = queue_io_deq_bits_data[4'hf:4'h8];
  assign T214 = queue_io_deq_bits_data[3'h7:1'h0];
  assign T215 = {T216, 24'h800000};
  assign T216 = queue_io_deq_bits_data[3'h7:1'h0];
  assign T217 = T67 & T218;
  assign T218 = 2'h1 == byteTrunc;
  assign T219 = {T220, 16'h8000};
  assign T220 = {T222, T221};
  assign T221 = queue_io_deq_bits_data[4'hf:4'h8];
  assign T222 = queue_io_deq_bits_data[3'h7:1'h0];
  assign T223 = T67 & T224;
  assign T224 = 2'h2 == byteTrunc;
  assign T225 = {T226, 8'h80};
  assign T226 = {T228, T227};
  assign T227 = queue_io_deq_bits_data[5'h17:5'h10];
  assign T228 = {T230, T229};
  assign T229 = queue_io_deq_bits_data[4'hf:4'h8];
  assign T230 = queue_io_deq_bits_data[3'h7:1'h0];
  assign T231 = T67 & T232;
  assign T232 = 2'h3 == byteTrunc;
  assign T233 = ml[6'h3f:6'h20];
  assign T234 = ml[5'h1f:1'h0];
  assign T235 = T257 | T500;
  assign T500 = {1'h0, T236};
  assign T236 = T237 << 1'h1;
  assign T237 = T238 & 31'h7fffffff;
  assign T238 = T239[5'h1e:1'h0];
  assign T239 = T255 ^ w15;
  assign T501 = reset ? 32'h0 : T240;
  assign T240 = T105 ? w14 : w15;
  assign T502 = reset ? 32'h0 : T241;
  assign T241 = T105 ? w13 : w14;
  assign T503 = reset ? 32'h0 : T242;
  assign T242 = T105 ? w12 : w13;
  assign T504 = reset ? 32'h0 : T243;
  assign T243 = T105 ? w11 : w12;
  assign T505 = reset ? 32'h0 : T244;
  assign T244 = T105 ? w10 : w11;
  assign T506 = reset ? 32'h0 : T245;
  assign T245 = T105 ? w9 : w10;
  assign T507 = reset ? 32'h0 : T246;
  assign T246 = T105 ? w8 : w9;
  assign T508 = reset ? 32'h0 : T247;
  assign T247 = T105 ? w7 : w8;
  assign T509 = reset ? 32'h0 : T248;
  assign T248 = T105 ? w6 : w7;
  assign T510 = reset ? 32'h0 : T249;
  assign T249 = T105 ? w5 : w6;
  assign T511 = reset ? 32'h0 : T250;
  assign T250 = T105 ? w4 : w5;
  assign T512 = reset ? 32'h0 : T251;
  assign T251 = T105 ? w3 : w4;
  assign T513 = reset ? 32'h0 : T252;
  assign T252 = T105 ? w2 : w3;
  assign T514 = reset ? 32'h0 : T253;
  assign T253 = T105 ? w1 : w2;
  assign T515 = reset ? 32'h0 : T254;
  assign T254 = T105 ? w0 : w1;
  assign T255 = T256 ^ w13;
  assign T256 = w2 ^ w7;
  assign T257 = T516 & T258;
  assign T258 = ~ T259;
  assign T259 = 33'hfffffffe;
  assign T516 = {1'h0, T200};
  assign T260 = T266 | T517;
  assign T517 = {T519, T261};
  assign T261 = T518 & T262;
  assign T262 = 2'h1;
  assign T518 = T263 ? 2'h3 : 2'h0;
  assign T263 = T264;
  assign T264 = T265;
  assign T265 = T239[5'h1f];
  assign T519 = T520 ? 31'h7fffffff : 31'h0;
  assign T520 = T261[1'h1];
  assign T266 = T199 & T521;
  assign T521 = {T522, T267};
  assign T267 = ~ T262;
  assign T522 = T523 ? 31'h7fffffff : 31'h0;
  assign T523 = T267[1'h1];
  assign T268 = memfill ^ 1'h1;
  assign T269 = T275 + k;
  assign k = T270;
  assign T270 = T114 ? 32'hca62c1d6 : T271;
  assign T271 = T112 ? 32'h8f1bbcdc : T272;
  assign T272 = T110 ? 32'h6ed9eba1 : T273;
  assign T273 = T108 ? 32'h5a827999 : T274;
  assign T274 = T115 ? 32'h5a827999 : 32'h0;
  assign T275 = T303 + e;
  assign T524 = reset ? 32'h0 : T276;
  assign T276 = T127 ? T300 : T277;
  assign T277 = T93 ? 32'hc3d2e1f0 : T278;
  assign T278 = T105 ? d : e;
  assign T525 = reset ? 32'h0 : T279;
  assign T279 = T127 ? T297 : T280;
  assign T280 = T93 ? 32'h10325476 : T281;
  assign T281 = T105 ? c : d;
  assign T526 = reset ? 32'h0 : T282;
  assign T282 = T127 ? T294 : T283;
  assign T283 = T93 ? 32'h98badcfe : T284;
  assign T284 = T105 ? T285 : c;
  assign T285 = {T293, T286};
  assign T286 = b[5'h1f:2'h2];
  assign T527 = reset ? 32'h0 : T287;
  assign T287 = T127 ? T290 : T288;
  assign T288 = T93 ? 32'hefcdab89 : T289;
  assign T289 = T105 ? a : b;
  assign T290 = h1 + b;
  assign T528 = reset ? 32'h0 : T291;
  assign T291 = T127 ? T290 : T292;
  assign T292 = T93 ? 32'hefcdab89 : h1;
  assign T293 = b[1'h1:1'h0];
  assign T294 = h2 + c;
  assign T529 = reset ? 32'h0 : T295;
  assign T295 = T127 ? T294 : T296;
  assign T296 = T93 ? 32'h98badcfe : h2;
  assign T297 = h3 + d;
  assign T530 = reset ? 32'h0 : T298;
  assign T298 = T127 ? T297 : T299;
  assign T299 = T93 ? 32'h10325476 : h3;
  assign T300 = h4 + e;
  assign T531 = reset ? 32'h0 : T301;
  assign T301 = T127 ? T300 : T302;
  assign T302 = T93 ? 32'hc3d2e1f0 : h4;
  assign T303 = T326 + f;
  assign f = T304;
  assign T304 = T114 ? T324 : T305;
  assign T305 = T112 ? T319 : T306;
  assign T306 = T110 ? T317 : T307;
  assign T307 = T108 ? T313 : T308;
  assign T308 = T115 ? T309 : 32'h0;
  assign T309 = T312 | T310;
  assign T310 = T311 & d;
  assign T311 = ~ b;
  assign T312 = b & c;
  assign T313 = T316 | T314;
  assign T314 = T315 & d;
  assign T315 = ~ b;
  assign T316 = b & c;
  assign T317 = T318 ^ d;
  assign T318 = b ^ c;
  assign T319 = T321 | T320;
  assign T320 = c & d;
  assign T321 = T323 | T322;
  assign T322 = b & d;
  assign T323 = b & c;
  assign T324 = T325 ^ d;
  assign T325 = b ^ c;
  assign T326 = {T328, T327};
  assign T327 = a[5'h1f:5'h1b];
  assign T328 = a[5'h1a:1'h0];
  assign T329 = {T331, T330};
  assign T330 = h0[5'h17:5'h10];
  assign T331 = {T333, T332};
  assign T332 = h0[4'hf:4'h8];
  assign T333 = h0[3'h7:1'h0];
  assign T334 = T186 & T335;
  assign T335 = 3'h3 == T185;
  assign T336 = {T338, T337};
  assign T337 = h1[5'h1f:5'h18];
  assign T338 = {T340, T339};
  assign T339 = h1[5'h17:5'h10];
  assign T340 = {T342, T341};
  assign T341 = h1[4'hf:4'h8];
  assign T342 = h1[3'h7:1'h0];
  assign T343 = T186 & T344;
  assign T344 = 3'h4 == T185;
  assign T345 = {T347, T346};
  assign T346 = h2[5'h1f:5'h18];
  assign T347 = {T349, T348};
  assign T348 = h2[5'h17:5'h10];
  assign T349 = {T351, T350};
  assign T350 = h2[4'hf:4'h8];
  assign T351 = h2[3'h7:1'h0];
  assign T352 = T186 & T353;
  assign T353 = 3'h5 == T185;
  assign T354 = {T356, T355};
  assign T355 = h3[5'h1f:5'h18];
  assign T356 = {T358, T357};
  assign T357 = h3[5'h17:5'h10];
  assign T358 = {T360, T359};
  assign T359 = h3[4'hf:4'h8];
  assign T360 = h3[3'h7:1'h0];
  assign T361 = T186 & T362;
  assign T362 = 3'h6 == T185;
  assign T363 = {T365, T364};
  assign T364 = h4[5'h1f:5'h18];
  assign T365 = {T367, T366};
  assign T366 = h4[5'h17:5'h10];
  assign T367 = {T369, T368};
  assign T368 = h4[4'hf:4'h8];
  assign T369 = h4[3'h7:1'h0];
  assign T370 = T186 & T371;
  assign T371 = 3'h7 == T185;
  assign T372 = T43 ? 1'h1 : T373;
  assign T373 = T374 ? 1'h1 : T92;
  assign T374 = T115 & gotWord;
  assign T375 = T421 ? 1'h1 : T376;
  assign T376 = T419 ? 1'h1 : T377;
  assign T377 = lastWord & queue_io_enq_ready;
  assign T532 = reset ? 1'h0 : T378;
  assign T378 = T25 ? 1'h0 : T379;
  assign T379 = T415 ? 1'h0 : T380;
  assign T380 = T413 ? 1'h0 : T381;
  assign T381 = T409 ? 1'h0 : T382;
  assign T382 = T407 ? 1'h0 : T383;
  assign T383 = T405 ? 1'h0 : T384;
  assign T384 = T401 ? 1'h0 : T385;
  assign T385 = T387 ? 1'h1 : T386;
  assign T386 = T377 ? 1'h0 : lastWord;
  assign T387 = T390 & T388;
  assign T388 = T389 ^ 1'h1;
  assign T389 = byteShift == 2'h0;
  assign T390 = T400 & T391;
  assign T391 = replyWordsLeft == 30'h1;
  assign T533 = reset ? 30'h0 : T392;
  assign T392 = T398 ? T141 : T393;
  assign T393 = T396 ? T140 : T394;
  assign T394 = T400 ? T395 : replyWordsLeft;
  assign T395 = replyWordsLeft - 30'h1;
  assign T396 = T25 & T397;
  assign T397 = byteTrunc == 2'h0;
  assign T398 = T25 & T399;
  assign T399 = T397 ^ 1'h1;
  assign T400 = T22 & io_in_reply;
  assign T401 = T403 & T402;
  assign T402 = byteShift == 2'h1;
  assign T403 = T387 & T404;
  assign T404 = byteTrunc == 2'h0;
  assign T405 = T403 & T406;
  assign T406 = byteShift == 2'h2;
  assign T407 = T403 & T408;
  assign T408 = byteShift == 2'h3;
  assign T409 = T411 & T410;
  assign T410 = byteShift == 2'h2;
  assign T411 = T387 & T412;
  assign T412 = byteTrunc == 2'h3;
  assign T413 = T411 & T414;
  assign T414 = byteShift == 2'h3;
  assign T415 = T417 & T416;
  assign T416 = byteShift == 2'h3;
  assign T417 = T387 & T418;
  assign T418 = byteTrunc == 2'h2;
  assign T419 = T400 & T420;
  assign T420 = replyWordsLeft == 30'h0;
  assign T421 = T390 & T389;
  assign T534 = T422[5'h1f:1'h0];
  assign T422 = T462 ? T542 : T423;
  assign T423 = T457 ? T541 : T424;
  assign T424 = T447 ? T539 : T425;
  assign T425 = T442 ? T538 : T426;
  assign T426 = T440 ? T537 : T427;
  assign T427 = T436 ? T536 : T428;
  assign T428 = T432 ? T429 : 48'h0;
  assign T429 = {24'h0, T430};
  assign T430 = storedData[5'h1f:4'h8];
  assign T535 = reset ? 32'h0 : T431;
  assign T431 = T400 ? io_in_data : storedData;
  assign T432 = T377 & T433;
  assign T433 = 2'h1 == byteShift;
  assign T536 = {16'h0, T434};
  assign T434 = {16'h0, T435};
  assign T435 = storedData[5'h1f:5'h10];
  assign T436 = T377 & T437;
  assign T437 = 2'h2 == byteShift;
  assign T537 = {32'h0, T438};
  assign T438 = {8'h0, T439};
  assign T439 = storedData[5'h1f:5'h18];
  assign T440 = T377 & T441;
  assign T441 = 2'h3 == byteShift;
  assign T538 = {16'h0, io_in_data};
  assign T442 = T400 & T443;
  assign T443 = byteShift == 2'h0;
  assign T539 = {16'h0, T444};
  assign T444 = {T446, T445};
  assign T445 = storedData[5'h1f:4'h8];
  assign T446 = io_in_data[3'h7:1'h0];
  assign T447 = T449 & T448;
  assign T448 = 2'h1 == byteShift;
  assign T449 = T400 & T450;
  assign T450 = T453 & storedDataValid;
  assign T540 = reset ? 1'h0 : T451;
  assign T451 = T25 ? 1'h0 : T452;
  assign T452 = T400 ? 1'h1 : storedDataValid;
  assign T453 = T443 ^ 1'h1;
  assign T541 = {16'h0, T454};
  assign T454 = {T456, T455};
  assign T455 = storedData[5'h1f:5'h10];
  assign T456 = io_in_data[4'hf:1'h0];
  assign T457 = T449 & T458;
  assign T458 = 2'h2 == byteShift;
  assign T542 = {16'h0, T459};
  assign T459 = {T461, T460};
  assign T460 = storedData[5'h1f:5'h18];
  assign T461 = io_in_data[5'h17:1'h0];
  assign T462 = T449 & T463;
  assign T463 = 2'h3 == byteShift;
  assign T464 = T449 ? 1'h1 : T465;
  assign T465 = T442 ? 1'h1 : T466;
  assign T466 = T440 ? 1'h1 : T467;
  assign T467 = T436 ? 1'h1 : T432;
  assign io_irq = interrupt;
  assign io_out_sender = T543;
  assign T543 = T468[4'h8:1'h0];
  assign T468 = replyQueueSha1_io_deq_valid ? replyQueueSha1_io_deq_bits_sender : 32'h0;
  assign io_out_data = T469;
  assign T469 = replyQueueSha1_io_deq_valid ? replyQueueSha1_io_deq_bits_data : 32'h0;
  assign io_out_exclusive = 1'h0;
  assign io_out_mask = 4'hf;
  assign io_out_write = 1'h0;
  assign io_out_reply = replyQueueSha1_io_deq_valid;
  assign io_out_burst = 1'h0;
  assign io_out_addr = T470;
  assign T470 = T163 ? addr : T471;
  assign T471 = replyQueueSha1_io_deq_valid ? replyQueueSha1_io_deq_bits_addr : T472;
  assign T472 = replyQueueSha1_io_deq_valid ? addr : io_in_addr;
  assign T544 = reset ? 32'h0 : T473;
  assign T473 = T162 ? T477 : T474;
  assign T474 = T148 ? T475 : addr;
  assign T475 = {T476, 2'h0};
  assign T476 = io_in_data[5'h1f:2'h2];
  assign T477 = addr + 32'h4;
  assign io_req_out = T478;
  assign T478 = T163 ? 1'h1 : replyQueueSha1_io_deq_valid;
  assign io_rdy_in = T479;
  assign T479 = queue_io_enq_ready & replyQueueSha1_io_enq_ready;
  QueueSha1_0 queue(.clk(clk), .reset(reset),
       .io_enq_ready( queue_io_enq_ready ),
       .io_enq_valid( T464 ),
       .io_enq_bits_data( T534 ),
       .io_enq_bits_last( T375 ),
       .io_deq_ready( T372 ),
       .io_deq_valid( queue_io_deq_valid ),
       .io_deq_bits_data( queue_io_deq_bits_data ),
       .io_deq_bits_last( queue_io_deq_bits_last )
       //.io_count(  )
  );
  QueueSha1_1 replyQueueSha1(.clk(clk), .reset(reset),
       .io_enq_ready( replyQueueSha1_io_enq_ready ),
       .io_enq_valid( T186 ),
       .io_enq_bits_data( T1 ),
       .io_enq_bits_addr( io_in_addr ),
       .io_enq_bits_sender( T480 ),
       .io_deq_ready( T0 ),
       .io_deq_valid( replyQueueSha1_io_deq_valid ),
       .io_deq_bits_data( replyQueueSha1_io_deq_bits_data ),
       .io_deq_bits_addr( replyQueueSha1_io_deq_bits_addr ),
       .io_deq_bits_sender( replyQueueSha1_io_deq_bits_sender )
       //.io_count(  )
  );

  always @(posedge clk) begin
    if(reset) begin
      interrupt <= 1'h0;
    end else if(T27) begin
      interrupt <= 1'h1;
    end else if(T183) begin
      interrupt <= 1'h0;
    end else if(T25) begin
      interrupt <= 1'h0;
    end else if(T11) begin
      interrupt <= 1'h1;
    end
    if(reset) begin
      ml <= 64'h0;
    end else if(T16) begin
      ml <= T14;
    end
    if(reset) begin
      chunksLeft <= 26'h0;
    end else if(T43) begin
      chunksLeft <= T42;
    end else if(T40) begin
      chunksLeft <= T39;
    end else if(T36) begin
      chunksLeft <= T32;
    end
    if(reset) begin
      memState <= 3'h0;
    end else if(T123) begin
      memState <= 3'h0;
    end else if(T121) begin
      memState <= 3'h4;
    end else if(T78) begin
      memState <= 3'h3;
    end else if(T76) begin
      memState <= 3'h2;
    end else if(T74) begin
      memState <= 3'h3;
    end else if(T70) begin
      memState <= 3'h1;
    end else if(T68) begin
      memState <= 3'h2;
    end else if(T60) begin
      memState <= 3'h3;
    end else if(T25) begin
      memState <= 3'h0;
    end
    if(reset) begin
      wordInChunk <= 4'h0;
    end else if(T119) begin
      wordInChunk <= T66;
    end else if(T117) begin
      wordInChunk <= T66;
    end else if(T48) begin
      wordInChunk <= T66;
    end else if(T25) begin
      wordInChunk <= 4'h0;
    end
    if(reset) begin
      byteTrunc <= 2'h0;
    end else if(T16) begin
      byteTrunc <= T73;
    end
    if(reset) begin
      hashState <= 3'h0;
    end else if(T43) begin
      hashState <= 3'h1;
    end else if(T27) begin
      hashState <= 3'h0;
    end else if(T113) begin
      hashState <= 3'h6;
    end else if(T111) begin
      hashState <= 3'h5;
    end else if(T109) begin
      hashState <= 3'h4;
    end else if(T96) begin
      hashState <= 3'h3;
    end else if(T116) begin
      hashState <= 3'h2;
    end else if(T92) begin
      hashState <= 3'h1;
    end
    if(reset) begin
      hashing <= 1'h0;
    end else if(T27) begin
      hashing <= 1'h0;
    end else if(T25) begin
      hashing <= 1'h1;
    end
    if(reset) begin
      hashCounter <= 5'h0;
    end else if(T125) begin
      hashCounter <= 5'hf;
    end else if(T111) begin
      hashCounter <= 5'h13;
    end else if(T109) begin
      hashCounter <= 5'h13;
    end else if(T96) begin
      hashCounter <= 5'h13;
    end else if(T116) begin
      hashCounter <= 5'h3;
    end else if(T93) begin
      hashCounter <= 5'hf;
    end else if(T105) begin
      hashCounter <= T104;
    end
    if(reset) begin
      prefetching <= 1'h0;
    end else if(T132) begin
      prefetching <= 1'h0;
    end else if(T25) begin
      prefetching <= 1'h1;
    end
    if(reset) begin
      pfWordsLeft <= 30'h0;
    end else if(T162) begin
      pfWordsLeft <= T161;
    end else if(T158) begin
      pfWordsLeft <= T156;
    end else if(T153) begin
      pfWordsLeft <= T156;
    end else if(T150) begin
      pfWordsLeft <= T156;
    end else if(T143) begin
      pfWordsLeft <= T141;
    end else if(T25) begin
      pfWordsLeft <= T140;
    end
    if(reset) begin
      byteShift <= 2'h0;
    end else if(T148) begin
      byteShift <= T147;
    end
    if(reset) begin
      pfCounter <= 6'h0;
    end else if(T162) begin
      pfCounter <= T174;
    end else if(pfNow) begin
      pfCounter <= T173;
    end else if(T171) begin
      pfCounter <= 6'h21;
    end else if(T169) begin
      pfCounter <= 6'h20;
    end
    if(reset) begin
      error <= 1'h0;
    end else if(T183) begin
      error <= 1'h0;
    end else if(T25) begin
      error <= 1'h0;
    end else if(T11) begin
      error <= 1'h1;
    end
    if(reset) begin
      h0 <= 32'h0;
    end else if(T127) begin
      h0 <= T193;
    end else if(T93) begin
      h0 <= 32'h67452301;
    end
    if(reset) begin
      a <= 32'h0;
    end else if(T127) begin
      a <= T193;
    end else if(T93) begin
      a <= 32'h67452301;
    end else if(T105) begin
      a <= T197;
    end
    w0 <= T497;
    if(reset) begin
      w15 <= 32'h0;
    end else if(T105) begin
      w15 <= w14;
    end
    if(reset) begin
      w14 <= 32'h0;
    end else if(T105) begin
      w14 <= w13;
    end
    if(reset) begin
      w13 <= 32'h0;
    end else if(T105) begin
      w13 <= w12;
    end
    if(reset) begin
      w12 <= 32'h0;
    end else if(T105) begin
      w12 <= w11;
    end
    if(reset) begin
      w11 <= 32'h0;
    end else if(T105) begin
      w11 <= w10;
    end
    if(reset) begin
      w10 <= 32'h0;
    end else if(T105) begin
      w10 <= w9;
    end
    if(reset) begin
      w9 <= 32'h0;
    end else if(T105) begin
      w9 <= w8;
    end
    if(reset) begin
      w8 <= 32'h0;
    end else if(T105) begin
      w8 <= w7;
    end
    if(reset) begin
      w7 <= 32'h0;
    end else if(T105) begin
      w7 <= w6;
    end
    if(reset) begin
      w6 <= 32'h0;
    end else if(T105) begin
      w6 <= w5;
    end
    if(reset) begin
      w5 <= 32'h0;
    end else if(T105) begin
      w5 <= w4;
    end
    if(reset) begin
      w4 <= 32'h0;
    end else if(T105) begin
      w4 <= w3;
    end
    if(reset) begin
      w3 <= 32'h0;
    end else if(T105) begin
      w3 <= w2;
    end
    if(reset) begin
      w2 <= 32'h0;
    end else if(T105) begin
      w2 <= w1;
    end
    if(reset) begin
      w1 <= 32'h0;
    end else if(T105) begin
      w1 <= w0;
    end
    if(reset) begin
      e <= 32'h0;
    end else if(T127) begin
      e <= T300;
    end else if(T93) begin
      e <= 32'hc3d2e1f0;
    end else if(T105) begin
      e <= d;
    end
    if(reset) begin
      d <= 32'h0;
    end else if(T127) begin
      d <= T297;
    end else if(T93) begin
      d <= 32'h10325476;
    end else if(T105) begin
      d <= c;
    end
    if(reset) begin
      c <= 32'h0;
    end else if(T127) begin
      c <= T294;
    end else if(T93) begin
      c <= 32'h98badcfe;
    end else if(T105) begin
      c <= T285;
    end
    if(reset) begin
      b <= 32'h0;
    end else if(T127) begin
      b <= T290;
    end else if(T93) begin
      b <= 32'hefcdab89;
    end else if(T105) begin
      b <= a;
    end
    if(reset) begin
      h1 <= 32'h0;
    end else if(T127) begin
      h1 <= T290;
    end else if(T93) begin
      h1 <= 32'hefcdab89;
    end
    if(reset) begin
      h2 <= 32'h0;
    end else if(T127) begin
      h2 <= T294;
    end else if(T93) begin
      h2 <= 32'h98badcfe;
    end
    if(reset) begin
      h3 <= 32'h0;
    end else if(T127) begin
      h3 <= T297;
    end else if(T93) begin
      h3 <= 32'h10325476;
    end
    if(reset) begin
      h4 <= 32'h0;
    end else if(T127) begin
      h4 <= T300;
    end else if(T93) begin
      h4 <= 32'hc3d2e1f0;
    end
    if(reset) begin
      lastWord <= 1'h0;
    end else if(T25) begin
      lastWord <= 1'h0;
    end else if(T415) begin
      lastWord <= 1'h0;
    end else if(T413) begin
      lastWord <= 1'h0;
    end else if(T409) begin
      lastWord <= 1'h0;
    end else if(T407) begin
      lastWord <= 1'h0;
    end else if(T405) begin
      lastWord <= 1'h0;
    end else if(T401) begin
      lastWord <= 1'h0;
    end else if(T387) begin
      lastWord <= 1'h1;
    end else if(T377) begin
      lastWord <= 1'h0;
    end
    if(reset) begin
      replyWordsLeft <= 30'h0;
    end else if(T398) begin
      replyWordsLeft <= T141;
    end else if(T396) begin
      replyWordsLeft <= T140;
    end else if(T400) begin
      replyWordsLeft <= T395;
    end
    if(reset) begin
      storedData <= 32'h0;
    end else if(T400) begin
      storedData <= io_in_data;
    end
    if(reset) begin
      storedDataValid <= 1'h0;
    end else if(T25) begin
      storedDataValid <= 1'h0;
    end else if(T400) begin
      storedDataValid <= 1'h1;
    end
    if(reset) begin
      addr <= 32'h0;
    end else if(T162) begin
      addr <= T477;
    end else if(T148) begin
      addr <= T475;
    end
  end
endmodule

