import uvm_pkg::*;
`include "uvm_macros.svh"
import sha1_pkg::*;
class sha1_driver extends uvm_driver#(sha1_transaction);
	`uvm_component_utils(sha1_driver);

	virtual sha1_if vif;

	function new(string name, uvm_component parent);
		super.new(name, parent);
	endfunction: new

	function void build_phase(uvm_phase phase);
		super.build_phase(phase);

		void'(uvm_resource_db#(virtual sha1_if)::read_by_name(.scope("ifs"), .name("sha1_if"), .val(vif)));
	endfunction: build_phase

	task run_phase(uvm_phase phase);
		drive();
	endtask: run_phase

	virtual task drive();
		sha1_transaction sha1_tx;
		
		string data_str; //TEST
		integer counter = 0, state = 0, addr = 0; //index = 0
		logic [31:0] data;
		vif.sig_rst = 1'b1;
		vif.sig_req_in = 1'b0;
        	vif.sig_in_addr = 32'b0;
	        vif.sig_in_reply = 1'b0;
        	vif.sig_in_burst = 1'b0;
        	vif.sig_in_sender = 9'b0;
	        vif.sig_in_write = 1'b0;
	        vif.sig_in_mask = 4'b0;
	        vif.sig_in_exclusive = 1'b0;
	        vif.sig_in_data = 32'b0;
	        vif.sig_rdy_out = 1'b0;
		
		@(posedge vif.sig_clk);
		@(posedge vif.sig_clk);
		vif.sig_rst = 1'b0;
		@(posedge vif.sig_clk);

 		forever begin

			@(posedge vif.sig_clk)
			begin
				case(state)
					0: begin
						seq_item_port.get_next_item(sha1_tx);
						`uvm_info("sha1_driver", sha1_tx.sprint(), UVM_LOW);						
						data_str = $sformatf("start_addr=%2d, size=%2d", sha1_tx.start_addr, sha1_tx.size);
						`uvm_info("sha1_driver", {"TRANSACTION: ",data_str}, UVM_LOW);
						data_str = $sformatf("Array: %2h, %2h, %2h, %2h, %2h, %2h, %2h, %2h, %2h, %2h, %2h, %2h, %2h, %2h, %2h, %2h", sha1_tx.input_array[0],
						sha1_tx.input_array[1], sha1_tx.input_array[2], sha1_tx.input_array[3], sha1_tx.input_array[4], sha1_tx.input_array[5], sha1_tx.input_array[6],
						sha1_tx.input_array[7], sha1_tx.input_array[8], sha1_tx.input_array[9], sha1_tx.input_array[10], sha1_tx.input_array[11],
						sha1_tx.input_array[12], sha1_tx.input_array[13], sha1_tx.input_array[14], sha1_tx.input_array[15]);
						`uvm_info("sha1_driver", {"TRANSACTION: ",data_str}, UVM_LOW);
						state = 1;
					end

					1: begin
						// write start address
					        vif.sig_req_in = 1'b1;
					        vif.sig_in_addr = 32'd0; //REG_ADDR
					        vif.sig_in_write = 1'b1;
					    	vif.sig_in_data = sha1_tx.start_addr * 4;
						data_str = $sformatf("%2d", vif.sig_in_data);
						`uvm_info("sha1_driver", {"REG_ADDR: ",data_str}, UVM_LOW);
						state = 2;
					end
					
					2: begin
						vif.sig_req_in = 1'b0;
						state = 3;
					end

					3: begin 
						// write size
					        vif.sig_req_in = 1'b1;
						vif.sig_in_addr = 32'd4; //REG_SIZE
					        vif.sig_in_write = 1'b1;
				    		vif.sig_in_data = sha1_tx.size;
						data_str = $sformatf("%2d", vif.sig_in_data);
						`uvm_info("sha1_driver", {"REG_SIZE: ",data_str}, UVM_LOW);
						state = 4;
					end
					
					4: begin
						vif.sig_req_in = 1'b0;
						state = 5;
					end	
	
					5: begin
						// initiate computation
						vif.sig_req_in = 1'b1;
					        vif.sig_in_addr = 32'd8; //REG_STATUS
					        vif.sig_in_write = 1'b1;
				    		vif.sig_in_data = 32'd1;
						state = 6;
					end
					
					6: begin
						vif.sig_req_in = 1'b0;
					   	state = 7;
					end	
					
					7: begin
						`uvm_info("sha1_driver", {"Computation started"}, UVM_LOW);
						vif.sig_req_in = 1'b0;
						vif.sig_rdy_out = 1'b1;
					   	state = 8;
					end	

					8: begin
						vif.sig_rdy_out = 1'b0;
						if (sha1_tx.size[1:0] == 2'b00 && counter == (sha1_tx.size/4))
							state = 11;	
						else if (sha1_tx.size[1:0] != 2'b00 && counter == ((sha1_tx.size/4) + 1))
							state = 11;	
						else if (vif.sig_req_out==1'b1)
							begin
							state = 9;
							addr = vif.sig_out_addr/4;
							data_str = $sformatf("%2d", addr);
							`uvm_info("sha1_driver", {"ADDR_REQ: ",data_str}, UVM_LOW);
							end
						else
							state = 8;
					end	

					9: begin
						vif.sig_rdy_out = 1'b0;
						data = sha1_tx.input_array[addr];
						vif.sig_in_data = {data[7:0],data[15:8],data[23:16],data[31:24]};
						data_str = $sformatf("%2h", vif.sig_in_data);
						`uvm_info("sha1_driver", {"INPUT_DATA: ",data_str}, UVM_LOW);
						vif.sig_in_write = 1'b0;
						vif.sig_req_in = 1'b1;
						vif.sig_in_reply = 1'b1;
						state = 10;
					end	

					10: begin
						vif.sig_rdy_out = 1'b1;
						vif.sig_in_reply = 1'b0;
						vif.sig_req_in = 1'b0;
						counter = counter + 1;
						data_str = $sformatf("%2d", counter);
						`uvm_info("sha1_driver", {"COUNTER: ",data_str}, UVM_LOW);
						state = 8;
					end	

					11: begin
						if(vif.sig_irq == 1'b1)
							begin
							state = 12;	
							counter = 0;
							`uvm_info("sha1_driver", {"Computation finished"}, UVM_LOW);
							end
						else					
							state = 11;
					end					
					
					12: begin
						if (counter == 5)
							state = 15;
						else
							begin
							vif.sig_req_in = 1'b1;
							case(counter)
						        	0: vif.sig_in_addr = 32'd12; //REG_H0
								1: vif.sig_in_addr = 32'd16; //REG_H1
								2: vif.sig_in_addr = 32'd20; //REG_H2
								3: vif.sig_in_addr = 32'd24; //REG_H3
								4: vif.sig_in_addr = 32'd28; //REG_H4
							endcase
						        vif.sig_in_write = 1'b0;
					    		vif.sig_in_reply = 1'b0;
							state = 13;
							end
					end	

					13: begin
						vif.sig_req_in = 1'b0;
						vif.sig_rdy_out = 1'b1;
					   	state = 14;
					end	

					14: begin
						vif.sig_rdy_out = 1'b0;
						if (vif.sig_req_out==1'b1)
							begin
							data = vif.sig_out_data;
							sha1_tx.hash_array[counter] = {data[7:0],data[15:8],data[23:16],data[31:24]};
							data_str = $sformatf("%2h", sha1_tx.hash_array[counter]);
							`uvm_info("sha1_driver", {"HASH: ",data_str}, UVM_LOW);
							counter = counter + 1;
							state = 12;
							end
						else
							state = 14;
					end

					15: begin
						state = 16;
						counter = 0;	
						vif.sig_rst = 1'b1;
						seq_item_port.item_done();
					end

					16: begin
						vif.sig_rst = 1'b0;
						state = 0;	
					end
				endcase
			end
		end 	 

       	endtask: drive
endclass: sha1_driver