import uvm_pkg::*;
`include "uvm_macros.svh"
import sha1_pkg::*;

parameter ARRAY_SIZE = 16;

class sha1_transaction extends uvm_sequence_item;
	rand logic [31:0] size ;
	rand logic [31:0] start_addr;
	rand logic [31:0] input_array [0:ARRAY_SIZE-1]; //16*32
	logic [31:0] hash_array [0:4];
	
	//constraints
	constraint size_c { size inside {[1:(ARRAY_SIZE*4)]};} //size in bytes
	constraint start_addr_c { 
		if (size[1:0] == 2'b00) {
			start_addr >= 0 && start_addr <= (ARRAY_SIZE - (size/4));	
		} else {
			start_addr >= 0 && start_addr <= (ARRAY_SIZE - (size/4) - 1);
		}	
	} 

	function new(string name = "");
		super.new(name);
	endfunction: new

	`uvm_object_utils(sha1_transaction)
	
endclass: sha1_transaction

class sha1_sequence extends uvm_sequence#(sha1_transaction);
	`uvm_object_utils(sha1_sequence)

	function new(string name = "");
		super.new(name);
	endfunction: new

	task body();
		sha1_transaction sha1_tx;
		
		repeat(10) begin
		sha1_tx = sha1_transaction::type_id::create(.name("sha1_tx"), .contxt(get_full_name()));

		start_item(sha1_tx);
		assert(sha1_tx.randomize());
		//`uvm_info("sha1_sequence", sha1_tx.sprint(), UVM_LOW);
		finish_item(sha1_tx);
		end
	endtask: body
endclass: sha1_sequence

typedef uvm_sequencer#(sha1_transaction) sha1_sequencer;
