import uvm_pkg::*;
`include "uvm_macros.svh"
import sha1_pkg::*;

class sha1_agent extends uvm_agent;
	`uvm_component_utils(sha1_agent)

	uvm_analysis_port#(sha1_transaction) agent_ap_minput;
	uvm_analysis_port#(sha1_transaction) agent_ap_moutput;

	sha1_agent_config   	sha1_agent_cfg;	
	sha1_sequencer		sha1_seqr;
	sha1_driver		sha1_drvr;
	sha1_monitor_input	sha1_mon_input;
	sha1_monitor_output	sha1_mon_output;

	function new(string name, uvm_component parent);
		super.new(name, parent);
	endfunction: new

	function void build_phase(uvm_phase phase);
		super.build_phase(phase);

		if ( ! uvm_config_db#( sha1_agent_config )::get
             		( .cntxt( this ), .inst_name( "" ), .field_name( "sha1_agent_config" ), .value( sha1_agent_cfg ) ) ) begin
         		`uvm_error( "sha1_agent", "sha1_agent_cfg not found" )
	      	end

		if ( sha1_agent_cfg.active == UVM_ACTIVE ) begin
         		sha1_seqr	 = sha1_sequencer::type_id::create(.name("sha1_seqr"), .parent(this));
         		sha1_drvr	 = sha1_driver::type_id::create(.name("sha1_drvr"), .parent(this));
      		end

		sha1_mon_input	 = sha1_monitor_input::type_id::create(.name("sha1_mon_input"), .parent(this));
		sha1_mon_output	 = sha1_monitor_output::type_id::create(.name("sha1_mon_output"), .parent(this));

		agent_ap_minput	 = new(.name("agent_ap_minput"), .parent(this));
		agent_ap_moutput = new(.name("agent_ap_moutput"), .parent(this));
		
	endfunction: build_phase

	function void connect_phase(uvm_phase phase);
		super.connect_phase(phase);
		
		if ( sha1_agent_cfg.active == UVM_ACTIVE ) begin		
			sha1_drvr.seq_item_port.connect(sha1_seqr.seq_item_export);
			sha1_drvr.vif = sha1_agent_cfg.vif;
		end

		sha1_mon_input.vif = sha1_agent_cfg.vif;
		sha1_mon_input.mon_ap_minput.connect(agent_ap_minput);
		sha1_mon_output.vif = sha1_agent_cfg.vif;		
		sha1_mon_output.mon_ap_moutput.connect(agent_ap_moutput);

	endfunction: connect_phase
endclass: sha1_agent





