`timescale 1ns/1ps
`include "common_defs.v"

// ============  interconnect data interface ===============

//`define ADDR_BEGIN     00   // address               - 32 bits
//`define ADDR_END       31   
//`define REPLY_IDX      32   // reply/request?        - 1 bit
//`define WR_REQ_IDX     33   // write request         - 1 bit
//`define WR_MASK_BEGIN  34   // write mask            - 4 bits
//`define WR_MASK_END    37
//`define EXCL_IDX       38   // exclusive operation   - 1 bit
//`define ERROR_IDX      39   // error                 - 1 bit
//`define BURST_IDX      40   // error                 - 1 bit
//`define DATA_BEGIN     41   // data                  - 32 bits
//`define DATA_END       72
//`define ID_IDX         73
//`define SEND_X_BEGIN   74   // sender coordinates    - 8 bits
//`define SEND_X_END     77
//`define SEND_Y_BEGIN   78
//`define SEND_Y_END     81
//`define DEST_X_BEGIN   82   // destination coordinates - 8 bits
//`define DEST_X_END     85   //       (IC only)
//`define DEST_Y_BEGIN   86
//`define DEST_Y_END     89

module sha1tile (clk, rst, routerE_req_in, routerE_data_in, routerE_rdy_in, routerE_req_out, routerE_data_out, routerE_rdy_out,
		router_mon_req_in, router_mon_data_in, router_mon_rdy_in, router_mon_req_out, router_mon_data_out, router_mon_rdy_out,
		SHA1_mon_req_in, SHA1_mon_in_addr, SHA1_mon_in_write, SHA1_mon_in_data, SHA1_mon_in_reply, SHA1_mon_req_out, SHA1_mon_out_data,
	    	SHA1_mon_irq);

// ==================== PARAMETER DEFINITION ====================
parameter LINKWIDTH = `IC_WIDTH;

parameter TILE_X = 4'd1; // x-coordinate of the tile
parameter TILE_Y = 4'd1; // y-coordinate of the tile

// ==================== INTERFACE ====================
input clk, rst;

input routerE_req_in;
input [LINKWIDTH-1:0] routerE_data_in;
output routerE_rdy_in;

output routerE_req_out;
output [LINKWIDTH-1:0] routerE_data_out;
input routerE_rdy_out;

// Router Monitoring
output [4:0] router_mon_req_in; 
output [LINKWIDTH*5-1:0] router_mon_data_in; 
output [4:0] router_mon_rdy_in;
output [4:0] router_mon_req_out;
output [LINKWIDTH*5-1:0] router_mon_data_out;
output [4:0] router_mon_rdy_out;

// SHA1 Monitoring
output SHA1_mon_req_in; 
output [31:0] SHA1_mon_in_addr;
output SHA1_mon_in_write;
output [31:0] SHA1_mon_in_data;
output SHA1_mon_in_reply;
output SHA1_mon_req_out;
output [31:0] SHA1_mon_out_data;
output SHA1_mon_irq;

// ==================== INTERNAL SIGNALS ====================
// Router signals
wire [4:0] router_req_in; 
wire [LINKWIDTH*5-1:0] router_data_in; 
wire [4:0] router_rdy_in;
wire [4:0] router_req_out;
wire [LINKWIDTH*5-1:0] router_data_out;
wire [4:0] router_rdy_out;

wire routerL_req_in; 
wire [LINKWIDTH-1:0] routerL_data_in; 
wire routerL_rdy_in;
wire routerL_req_out;
wire [LINKWIDTH-1:0] routerL_data_out;
wire routerL_rdy_out;

// SHA1 signals
wire SHA1_req_in;
wire SHA1_rdy_in;
wire [31:0] SHA1_in_addr;
wire SHA1_in_burst;
wire SHA1_in_reply;
wire SHA1_in_write;
wire [3:0] SHA1_in_mask;
wire SHA1_in_exclusive;
wire [31:0] SHA1_in_data;
wire [8:0] SHA1_in_sender;
wire SHA1_req_out;
wire SHA1_rdy_out;
wire [31:0] SHA1_out_addr;
wire SHA1_out_burst;
wire SHA1_out_reply;
wire SHA1_out_write;
wire [3:0] SHA1_out_mask;
//wire SHA1_out_exclusive;
wire [31:0] SHA1_out_data;
wire [8:0] SHA1_out_sender;
wire SHA1_irq;

// =============== INTERNAL LOGIC ==================
// Router Local Ports Input&Output
assign router_req_in[`LOCAL_DIR] = 1'b0 ; //change  routerL_req_in
assign router_data_in[LINKWIDTH*(`LOCAL_DIR+1)-1:LINKWIDTH*`LOCAL_DIR] = 0;  //change routerL_data_in
assign routerL_rdy_in = router_rdy_in[`LOCAL_DIR];

assign routerL_req_out = router_req_out[`LOCAL_DIR]; 
assign routerL_data_out = router_data_out[LINKWIDTH*(`LOCAL_DIR+1)-1:LINKWIDTH*`LOCAL_DIR];
assign router_rdy_out[`LOCAL_DIR] = routerL_rdy_out;

// Router East Ports Input&Output
assign router_req_in[`EAST_DIR] = routerE_req_in;
assign router_data_in[LINKWIDTH*(`EAST_DIR+1)-1:LINKWIDTH*`EAST_DIR] = routerE_data_in;  
assign routerE_rdy_in = router_rdy_in[`EAST_DIR];

assign routerE_req_out = routerL_req_in;   //change router_req_out[`EAST_DIR]
assign routerE_data_out = routerL_data_in; //change router_data_out[LINKWIDTH*(`EAST_DIR+1)-1:LINKWIDTH*`EAST_DIR]
assign router_rdy_out[`EAST_DIR] = 1'b1; //change

// Router West Ports Input&Output
assign router_req_in[`WEST_DIR] = 1'b0;
assign router_data_in[LINKWIDTH*(`WEST_DIR+1)-1:LINKWIDTH*`WEST_DIR] = 0;  
assign router_rdy_out[`WEST_DIR] = 1'b1;

// Router North Ports Input&Output
assign router_req_in[`NORTH_DIR] = 1'b0;
assign router_data_in[LINKWIDTH*(`NORTH_DIR+1)-1:LINKWIDTH*`NORTH_DIR] = 0;  
assign router_rdy_out[`NORTH_DIR] = 1'b1;

// Router South Ports Input&Output
assign router_req_in[`SOUTH_DIR] = 1'b0;
assign router_data_in[LINKWIDTH*(`SOUTH_DIR+1)-1:LINKWIDTH*`SOUTH_DIR] = 0;  
assign router_rdy_out[`SOUTH_DIR] = 1'b1;

// SHA1 connections
assign SHA1_req_in = routerL_req_out;
assign SHA1_in_addr = routerL_data_out[`ADDR_END:`ADDR_BEGIN];
assign SHA1_in_burst = routerL_data_out[`BURST_IDX];
assign SHA1_in_reply = routerL_data_out[`REPLY_IDX];
assign SHA1_in_write = routerL_data_out[`WR_REQ_IDX];
assign SHA1_in_mask = routerL_data_out[`WR_MASK_END:`WR_MASK_BEGIN];
assign SHA1_in_exclusive = routerL_data_out[`EXCL_IDX];
assign SHA1_in_data = routerL_data_out[`DATA_END:`DATA_BEGIN];
assign SHA1_in_sender = routerL_data_out[`SEND_Y_END:`ID_IDX];
assign routerL_req_in = SHA1_req_out;
assign SHA1_rdy_out = routerL_rdy_in & routerE_rdy_out; //change
assign routerL_data_in[`ADDR_END:`ADDR_BEGIN] = SHA1_out_addr;
assign routerL_data_in[`BURST_IDX] = SHA1_out_burst;
assign routerL_data_in[`REPLY_IDX] = SHA1_out_reply;
assign routerL_data_in[`WR_REQ_IDX] = SHA1_out_write;
assign routerL_data_in[`WR_MASK_END:`WR_MASK_BEGIN] = SHA1_out_mask;
//assign routerL_data_in[`EXCL_IDX] = SHA1_out_exclusive;
assign routerL_data_in[`DATA_END:`DATA_BEGIN] = SHA1_out_data;
assign routerL_data_in[`SEND_Y_END:`ID_IDX] = SHA1_out_sender;
assign routerL_rdy_out = SHA1_rdy_in;
assign routerL_data_in[`EXCL_IDX] = SHA1_irq;
assign routerL_data_in[`ERROR_IDX] = 1'b0;
assign routerL_data_in[`DEST_X_END:`DEST_X_BEGIN] = 4'd2; //EAST DIRECTION
assign routerL_data_in[`DEST_Y_END:`DEST_Y_BEGIN] = 4'd0;

// =========== MODULES INSTANTIATION =================

// Router instantiation
router dut1 (	.clk (clk),
         	.rst (rst),
		.req_in (router_req_in),
             	.data_in (router_data_in),
             	.rdy_in (router_rdy_in),
		.req_out (router_req_out),
		.data_out (router_data_out),
             	.rdy_out (router_rdy_out)
);

Sha1 dut2 (	.clk (clk),
            	.reset (rst),
		.io_req_in (SHA1_req_in),
             	.io_rdy_in (SHA1_rdy_in),
             	.io_in_addr (SHA1_in_addr),
             	.io_in_burst (SHA1_in_burst),
             	.io_in_reply (SHA1_in_reply),
	        .io_in_write (SHA1_in_write),
	        .io_in_mask (SHA1_in_mask),
             	.io_in_exclusive (SHA1_in_exclusive),
             	.io_in_data (SHA1_in_data),
             	.io_in_sender (SHA1_in_sender),
             	.io_req_out (SHA1_req_out),
             	.io_rdy_out (SHA1_rdy_out),
             	.io_out_addr (SHA1_out_addr),
             	.io_out_burst (SHA1_out_burst),
             	.io_out_reply (SHA1_out_reply),
             	.io_out_write (SHA1_out_write),
            	.io_out_mask (SHA1_out_mask),
             	.io_out_exclusive (),
             	.io_out_data (SHA1_out_data),
             	.io_out_sender (SHA1_out_sender),
             	.io_irq (SHA1_irq)
);

assign router_mon_req_in = router_req_in; 
assign router_mon_data_in = router_data_in; 
assign router_mon_rdy_in = router_rdy_in;
assign router_mon_req_out = router_req_out;
assign router_mon_data_out = router_data_out;
assign router_mon_rdy_out = router_rdy_out;

assign SHA1_mon_req_in = SHA1_req_in; 
assign SHA1_mon_in_addr = SHA1_in_addr;
assign SHA1_mon_in_write = SHA1_in_write;
assign SHA1_mon_in_data = SHA1_in_data;
assign SHA1_mon_in_reply = SHA1_in_reply;
assign SHA1_mon_req_out = SHA1_req_out;
assign SHA1_mon_out_data = SHA1_out_data;
assign SHA1_mon_irq = SHA1_irq;

endmodule
