import uvm_pkg::*;
`include "uvm_macros.svh"
import sha1tile_pkg::*;

class sha1tile_agent extends uvm_agent;
	`uvm_component_utils(sha1tile_agent)

	uvm_analysis_port#(sha1tile_transaction) agent_ap_minput;
	uvm_analysis_port#(sha1tile_transaction) agent_ap_moutput;

	sha1tile_agent_config   	sha1tile_agent_cfg;	
	sha1tile_sequencer		sha1tile_seqr;
	sha1tile_driver			sha1tile_drvr;
	sha1tile_monitor_input		sha1tile_mon_input;
	sha1tile_monitor_output		sha1tile_mon_output;

	function new(string name, uvm_component parent);
		super.new(name, parent);
	endfunction: new

	function void build_phase(uvm_phase phase);
		super.build_phase(phase);

		if ( ! uvm_config_db#( sha1tile_agent_config )::get
             		( .cntxt( this ), .inst_name( "" ), .field_name( "sha1tile_agent_config" ), .value( sha1tile_agent_cfg ) ) ) begin
         		`uvm_error( "sha1tile_agent", "sha1tile_agent_cfg not found" )
	      	end

		if ( sha1tile_agent_cfg.active == UVM_ACTIVE ) begin
         		sha1tile_seqr	 = sha1tile_sequencer::type_id::create(.name("sha1tile_seqr"), .parent(this));
         		sha1tile_drvr	 = sha1tile_driver::type_id::create(.name("sha1tile_drvr"), .parent(this));
      		end

		sha1tile_mon_input	 = sha1tile_monitor_input::type_id::create(.name("sha1tile_mon_input"), .parent(this));
		sha1tile_mon_output	 = sha1tile_monitor_output::type_id::create(.name("sha1tile_mon_output"), .parent(this));

		agent_ap_minput	 = new(.name("agent_ap_minput"), .parent(this));
		agent_ap_moutput = new(.name("agent_ap_moutput"), .parent(this));
		
	endfunction: build_phase

	function void connect_phase(uvm_phase phase);
		super.connect_phase(phase);
		
		if ( sha1tile_agent_cfg.active == UVM_ACTIVE ) begin		
			sha1tile_drvr.seq_item_port.connect(sha1tile_seqr.seq_item_export);
			sha1tile_drvr.vif = sha1tile_agent_cfg.vif;
		end

		sha1tile_mon_input.vif = sha1tile_agent_cfg.vif;
		sha1tile_mon_input.mon_ap_minput.connect(agent_ap_minput);
		sha1tile_mon_output.vif = sha1tile_agent_cfg.vif;		
		sha1tile_mon_output.mon_ap_moutput.connect(agent_ap_moutput);

	endfunction: connect_phase
endclass: sha1tile_agent



