import uvm_pkg::*;
`include "uvm_macros.svh"
`include "common_defs.v"
import sha1tile_pkg::*;
class sha1tile_driver extends uvm_driver#(sha1tile_transaction);
	`uvm_component_utils(sha1tile_driver);

	virtual sha1tile_if vif;

	function new(string name, uvm_component parent);
		super.new(name, parent);
	endfunction: new

	function void build_phase(uvm_phase phase);
		super.build_phase(phase);

		void'(uvm_resource_db#(virtual sha1tile_if)::read_by_name(.scope("ifs"), .name("sha1tile_if"), .val(vif)));
	endfunction: build_phase

	task run_phase(uvm_phase phase);
		drive();
	endtask: run_phase

	virtual task drive();
		sha1tile_transaction sha1tile_tx;	

		string data_str; //TEST
		integer counter = 0, state = 0, addr = 0; //index = 0
		logic [31:0] out_addr;
		logic [31:0] data;
	   	logic [31:0] in_addr = 32'b0;
	        logic in_reply = 1'b0;
		logic in_write = 1'b0;
		logic [3:0] in_mask = 4'b0;
		logic in_exclusive = 1'b0;
		logic in_error = 1'b0;
		logic in_burst = 1'b0;
		logic [31:0] in_data = 32'b0;
		logic in_id = 1'b0;
		logic [3:0] send_x = 4'd2; //EAST
		logic [3:0] send_y = 4'd0;
		logic [3:0] dest_x = 4'd1; //LOCAL
		logic [3:0] dest_y = 4'd1;
        	
		vif.sig_rst = 1'b1;
		vif.sig_req_in = 1'b0;
		vif.sig_rdy_out = 1'b0;
	        vif.sig_data_in = {dest_y, dest_x, send_y, send_x, in_id, in_data, in_burst,
				   in_error, in_exclusive, in_mask, in_write, in_reply, in_addr};
	     
		@(posedge vif.sig_clk);
		@(posedge vif.sig_clk);
		vif.sig_rst = 1'b0;
		@(posedge vif.sig_clk);

 		forever begin

			@(posedge vif.sig_clk)
			begin
				case(state)
					0: begin
						seq_item_port.get_next_item(sha1tile_tx);
						`uvm_info("sha1tile_driver", sha1tile_tx.sprint(), UVM_LOW);						
						data_str = $sformatf("start_addr=%2d, size=%2d", sha1tile_tx.start_addr, sha1tile_tx.size);
						`uvm_info("sha1tile_driver", {"TRANSACTION: ",data_str}, UVM_LOW);
						data_str = $sformatf("Array: %2h, %2h, %2h, %2h, %2h, %2h, %2h, %2h, %2h, %2h, %2h, %2h, %2h, %2h, %2h, %2h", sha1tile_tx.input_array[0],
						sha1tile_tx.input_array[1], sha1tile_tx.input_array[2], sha1tile_tx.input_array[3], sha1tile_tx.input_array[4], sha1tile_tx.input_array[5], sha1tile_tx.input_array[6],
						sha1tile_tx.input_array[7], sha1tile_tx.input_array[8], sha1tile_tx.input_array[9], sha1tile_tx.input_array[10], sha1tile_tx.input_array[11],
						sha1tile_tx.input_array[12], sha1tile_tx.input_array[13], sha1tile_tx.input_array[14], sha1tile_tx.input_array[15]);
						`uvm_info("sha1tile_driver", {"TRANSACTION: ",data_str}, UVM_LOW);
						state = 1;
					end

					1: begin
						// write start address
					        vif.sig_req_in = 1'b1;
					        in_addr = 32'd0; //REG_ADDR
					        in_write = 1'b1;
					    	in_data = sha1tile_tx.start_addr * 4;
						data_str = $sformatf("%2d", in_data);
						`uvm_info("sha1tile_driver", {"REG_ADDR: ",data_str}, UVM_LOW);
						vif.sig_data_in = {dest_y, dest_x, send_y, send_x, in_id, in_data, in_burst,
				   		in_error, in_exclusive, in_mask, in_write, in_reply, in_addr};
						state = 2;
					end
					
					2: begin
						vif.sig_req_in = 1'b0;
						state = 3;
					end

					3: begin 
						// write size
					        vif.sig_req_in = 1'b1;
						in_addr = 32'd4; //REG_SIZE
					        in_write = 1'b1;
				    		in_data = sha1tile_tx.size;
						data_str = $sformatf("%2d", in_data);
						`uvm_info("sha1tile_driver", {"REG_SIZE: ",data_str}, UVM_LOW);
						vif.sig_data_in = {dest_y, dest_x, send_y, send_x, in_id, in_data, in_burst,
				   		in_error, in_exclusive, in_mask, in_write, in_reply, in_addr};
						state = 4;
					end
					
					4: begin
						vif.sig_req_in = 1'b0;
						state = 5;
					end	
	
					5: begin
						// initiate computation
						vif.sig_req_in = 1'b1;
					        in_addr = 32'd8; //REG_STATUS
					        in_write = 1'b1;
				    		in_data = 32'd1;
						vif.sig_data_in = {dest_y, dest_x, send_y, send_x, in_id, in_data, in_burst,
				   		in_error, in_exclusive, in_mask, in_write, in_reply, in_addr};
						state = 6;
					end
					
					6: begin
						vif.sig_req_in = 1'b0;
					   	state = 7;
					end	
					
					7: begin
						`uvm_info("sha1tile_driver", {"Computation started"}, UVM_LOW);
						vif.sig_req_in = 1'b0;
						vif.sig_rdy_out = 1'b1; //changed
					   	state = 8;
					end	

					8: begin
						vif.sig_rdy_out = 1'b0; //changed
						if (sha1tile_tx.size[1:0] == 2'b00 && counter == (sha1tile_tx.size/4))
							state = 11;	
						else if (sha1tile_tx.size[1:0] != 2'b00 && counter == ((sha1tile_tx.size/4) + 1))
							state = 11;	
						else if (vif.sig_req_out==1'b1)
							begin
							state = 9;
							out_addr = vif.sig_data_out[`ADDR_END:`ADDR_BEGIN];  
							addr = out_addr/4;
							data_str = $sformatf("%2d", addr);
							`uvm_info("sha1tile_driver", {"ADDR_REQ: ",data_str}, UVM_LOW);
							end
						else
							state = 8;
					end	

					9: begin
						vif.sig_req_in = 1'b1;
						vif.sig_rdy_out = 1'b0;
						data = sha1tile_tx.input_array[addr];
						in_data = {data[7:0],data[15:8],data[23:16],data[31:24]};
						in_write = 1'b0;
						in_reply = 1'b1;
						vif.sig_data_in = {dest_y, dest_x, send_y, send_x, in_id, in_data, in_burst,
				   		in_error, in_exclusive, in_mask, in_write, in_reply, in_addr};
						data_str = $sformatf("%2h", in_data);
						`uvm_info("sha1tile_driver", {"INPUT_DATA: ",data_str}, UVM_LOW);
						data_str = $sformatf("%b", vif.sig_data_in);
						`uvm_info("sha1tile_driver", {"DATA_IN: ",data_str}, UVM_LOW);
						state = 10;
					end	

					10: begin
						vif.sig_rdy_out = 1'b0;  //changed
						in_reply = 1'b0;
						vif.sig_data_in = {dest_y, dest_x, send_y, send_x, in_id, in_data, in_burst,
				   		in_error, in_exclusive, in_mask, in_write, in_reply, in_addr};
						vif.sig_req_in = 1'b0;
						counter = counter + 1;
						data_str = $sformatf("%2d", counter);
						`uvm_info("sha1tile_driver", {"COUNTER: ",data_str}, UVM_LOW);
						state = 20;
					end	
					
					
					20: begin
						vif.sig_rdy_out = 1'b1;
						state = 8;
					end	


					11: begin
						if(vif.sig_data_out[`EXCL_IDX] == 1'b1) //irq
							begin
							state = 12;	
							counter = 0;
							`uvm_info("sha1tile_driver", {"Computation finished"}, UVM_LOW);
							end
						else					
							state = 11;
					end					
					
					12: begin
						if (counter == 5)
							state = 15;
						else
							begin
							vif.sig_req_in = 1'b1;
							case(counter)
						        	0: in_addr = 32'd12; //REG_H0
								1: in_addr = 32'd16; //REG_H1
								2: in_addr = 32'd20; //REG_H2
								3: in_addr = 32'd24; //REG_H3
								4: in_addr = 32'd28; //REG_H4
							endcase
						        in_write = 1'b0;
					    		in_reply = 1'b0;
							vif.sig_data_in = {dest_y, dest_x, send_y, send_x, in_id, in_data, in_burst,
				   			in_error, in_exclusive, in_mask, in_write, in_reply, in_addr};
							state = 13;
							end
					end	

					13: begin
						vif.sig_req_in = 1'b0;
						vif.sig_rdy_out = 1'b1;
					   	state = 14;
					end	

					14: begin
						vif.sig_rdy_out = 1'b1;
						if (vif.sig_req_out==1'b1)
							begin
							data = vif.sig_data_out[`DATA_END:`DATA_BEGIN];
							sha1tile_tx.hash_array[counter] = {data[7:0],data[15:8],data[23:16],data[31:24]};
							data_str = $sformatf("%2h", sha1tile_tx.hash_array[counter]);
							`uvm_info("sha1tile_driver", {"HASH: ",data_str}, UVM_LOW);
							counter = counter + 1;
							state = 12;
							end
						else
							state = 14;
					end

					15: begin
						state = 16;
						counter = 0;	
						vif.sig_rst = 1'b1;
						seq_item_port.item_done();
					end

					16: begin
						vif.sig_rst = 1'b0;
						state = 0;	
					end
				endcase
			end
		end 	 

       	endtask: drive
endclass: sha1tile_driver
