import uvm_pkg::*;
`include "uvm_macros.svh"
import sha1tiletest_pkg::*;

class sha1tile_env extends uvm_env;
	`uvm_component_utils(sha1tile_env)

	sha1tile_env_config 	sha1tile_env_cfg;
	router_s_agent 		router_agnt;
	sha1_agent 		sha1_agnt;
	sha1tile_agent 		sha1tile_agnt;
	router_s_scoreboard 	router_sb;
	sha1_scoreboard 	sha1_sb;
	sha1tile_scoreboard 	sha1tile_sb;

	function new(string name, uvm_component parent);
		super.new(name, parent);
	endfunction: new

	function void build_phase(uvm_phase phase);
		super.build_phase(phase);
		
		if ( ! uvm_config_db#( sha1tile_env_config )::get
            		( .cntxt( this ), .inst_name( "" ), .field_name( "sha1tile_env_cfg" ), .value( sha1tile_env_cfg ) ) ) begin
         		`uvm_error( "sha1tile_env", "sha1tile_env_cfg not found" )
      		end
 
      		if ( sha1tile_env_cfg.has_router_s_agent ) begin
        		uvm_config_db#( router_s_agent_config )::set
	            	( .cntxt( this ), .inst_name( "router_agnt" ), .field_name( "router_s_agent_config" ), .value( sha1tile_env_cfg.router_s_agent_cfg ) );
         		router_agnt = router_s_agent::type_id::create( .name( "router_agnt" ), .parent( this ) );
 
         		if ( sha1tile_env_cfg.has_router_s_sb ) begin
            			router_sb = router_s_scoreboard::type_id::create( .name( "router_sb" ), .parent( this ) );
        		end
      		end
 	
		if ( sha1tile_env_cfg.has_sha1_agent ) begin
        		uvm_config_db#( sha1_agent_config )::set
	            	( .cntxt( this ), .inst_name( "sha1_agnt" ), .field_name( "sha1_agent_config" ), .value( sha1tile_env_cfg.sha1_agent_cfg ) );
         		sha1_agnt = sha1_agent::type_id::create(.name("sha1_agnt"), .parent(this));
 
         		if ( sha1tile_env_cfg.has_sha1_sb ) begin
            			sha1_sb	= sha1_scoreboard::type_id::create(.name("sha1_sb"), .parent(this));    
			end    		
      		end	
   
		if ( sha1tile_env_cfg.has_sha1tile_agent ) begin
        		uvm_config_db#( sha1tile_agent_config )::set
	            	( .cntxt( this ), .inst_name( "sha1tile_agnt" ), .field_name( "sha1tile_agent_config" ), .value( sha1tile_env_cfg.sha1tile_agent_cfg ) );
         		sha1tile_agnt = sha1tile_agent::type_id::create(.name("sha1tile_agnt"), .parent(this));
 
         		if ( sha1tile_env_cfg.has_sha1tile_sb ) begin
            			sha1tile_sb = sha1tile_scoreboard::type_id::create(.name("sha1tile_sb"), .parent(this)); 
			end       		
      		end	
		
	endfunction: build_phase

	function void connect_phase(uvm_phase phase);
		super.connect_phase(phase);
		
		if ( sha1tile_env_cfg.has_router_s_agent && sha1tile_env_cfg.has_router_s_sb ) begin
			router_agnt.agent_ap_minput.connect(router_sb.sb_export_minput);
			router_agnt.agent_ap_moutput.connect(router_sb.sb_export_moutput);
		end
     	 	
		if ( sha1tile_env_cfg.has_sha1_agent && sha1tile_env_cfg.has_sha1_sb ) begin
			sha1_agnt.agent_ap_minput.connect(sha1_sb.sb_export_minput);
			sha1_agnt.agent_ap_moutput.connect(sha1_sb.sb_export_moutput);	
		end
	
		if ( sha1tile_env_cfg.has_sha1tile_agent && sha1tile_env_cfg.has_sha1tile_sb ) begin
			sha1tile_agnt.agent_ap_minput.connect(sha1tile_sb.sb_export_minput);
			sha1tile_agnt.agent_ap_moutput.connect(sha1tile_sb.sb_export_moutput);	
		end

	endfunction: connect_phase
endclass: sha1tile_env


