import uvm_pkg::*;
`include "uvm_macros.svh"
`include "common_defs.v"
import sha1tile_pkg::*;
class sha1tile_monitor_input extends uvm_monitor;
	`uvm_component_utils(sha1tile_monitor_input);

	uvm_analysis_port#(sha1tile_transaction) mon_ap_minput;

	virtual sha1tile_if vif;

	sha1tile_transaction sha1tile_tx;
	
	//For coverage
	sha1tile_transaction sha1tile_tx_cg;

	//Define coverpoints
	covergroup sha1tile_cg;
      		size_cp:     	coverpoint sha1tile_tx_cg.size;
      		start_addr_cp:  coverpoint sha1tile_tx_cg.start_addr;
		cross size_cp, start_addr_cp;
	endgroup: sha1tile_cg

	function new(string name, uvm_component parent);
		super.new(name, parent);
		sha1tile_cg = new;
	endfunction: new

	function void build_phase(uvm_phase phase);
		super.build_phase(phase);

		void'(uvm_resource_db#(virtual sha1tile_if)::read_by_name(.scope("ifs"), .name("sha1tile_if"), .val(vif)));
		mon_ap_minput = new(.name("mon_ap_minput"), .parent(this));
	endfunction: build_phase

	task run_phase(uvm_phase phase);
		integer counter = 0, state = 0, size = 0;
		sha1tile_tx = sha1tile_transaction::type_id::create(.name("sha1tile_tx"), .contxt(get_full_name()));

		forever begin

			@(posedge vif.sig_clk)
			begin
				case(state)
					0: begin
						if (vif.sig_req_in == 1'b1 && vif.sig_data_in[`WR_REQ_IDX] == 1'b1 && vif.sig_data_in[`ADDR_END:`ADDR_BEGIN] == 32'd0) //REG_ADDR
							begin	 
							sha1tile_tx.start_addr = vif.sig_data_in[`DATA_END:`DATA_BEGIN];
							state = 1; 	
							end
						else
							state = 0;
					end

					1: begin
						if (vif.sig_req_in == 1'b1 && vif.sig_data_in[`WR_REQ_IDX] == 1'b1 && vif.sig_data_in[`ADDR_END:`ADDR_BEGIN] == 32'd4) //REG_SIZE
							begin	 
							size = vif.sig_data_in[`DATA_END:`DATA_BEGIN];
							sha1tile_tx.size = vif.sig_data_in[`DATA_END:`DATA_BEGIN];
							state = 2; 	
							end
						else
							state = 1;
					end
					
					2: begin
						if (vif.sig_req_in == 1'b1)
							state = 3;
						else
							state = 2;
					end

					3: begin
						if (size[1:0] == 2'b00 && counter == (size/4))
							state = 4;	
						else if (size[1:0] != 2'b00 && counter == ((size/4) + 1))
							state = 4;							
						else if (vif.sig_req_in == 1'b1 && vif.sig_data_in[`REPLY_IDX] == 1'b1)
							begin
							sha1tile_tx.input_array[counter] = vif.sig_data_in[`DATA_END:`DATA_BEGIN];
							counter = counter + 1;
							state = 3;
							end
						else
							state = 3;
					end

					4: begin
						state = 0;
						counter = 0;
						//Predict the result
						predictor();
						sha1tile_tx_cg = sha1tile_tx;
						//Coverage
						sha1tile_cg.sample();
						//Send the transaction to the analysis port
						mon_ap_minput.write(sha1tile_tx);
					end
				endcase
			end
		end
	endtask: run_phase

	virtual function void predictor();
		sha1tile_tx.hash_array[0] = 32'h273c4091;   // DUMMY CODE
		sha1tile_tx.hash_array[1] = 32'h47b8c4f8;
		sha1tile_tx.hash_array[2] = 32'h88278905;
		sha1tile_tx.hash_array[3] = 32'h4d22c93b;
		sha1tile_tx.hash_array[4] = 32'h830479b9;
	endfunction: predictor

endclass: sha1tile_monitor_input

class sha1tile_monitor_output extends uvm_monitor;
	`uvm_component_utils(sha1tile_monitor_output)

	uvm_analysis_port#(sha1tile_transaction) mon_ap_moutput;

	virtual sha1tile_if vif;
	
	function new(string name, uvm_component parent);
		super.new(name, parent);
	endfunction: new

	function void build_phase(uvm_phase phase);
		super.build_phase(phase);

		void'(uvm_resource_db#(virtual sha1tile_if)::read_by_name(.scope("ifs"), .name("sha1tile_if"), .val(vif)));
		mon_ap_moutput= new(.name("mon_ap_moutput"), .parent(this));
	endfunction: build_phase

	task run_phase(uvm_phase phase);
		integer counter = 0, state = 0;
		logic [31:0] data;

		sha1tile_transaction sha1tile_tx;
		sha1tile_tx = sha1tile_transaction::type_id::create(.name("sha1tile_tx"), .contxt(get_full_name()));

		forever begin
			@(posedge vif.sig_clk)
			begin
				case(state)
					0: begin
						if(vif.sig_data_out[`EXCL_IDX] == 1'b1) //irq
							state = 1;
						else
							state = 0;
					end

					1: begin
						if (counter == 5)
							state = 2;
						else if (vif.sig_req_out==1'b1)
							begin
							data = vif.sig_data_out[`DATA_END:`DATA_BEGIN];
							sha1tile_tx.hash_array[counter] = {data[7:0],data[15:8],data[23:16],data[31:24]};
							counter = counter + 1;
							state = 1;
							end
						else
							state = 1;
					end
					
					2: begin
						state = 3;
						counter = 0;
						//Send the transaction to the analysis port
						mon_ap_moutput.write(sha1tile_tx);
					end

					3: begin
						state = 0;
					end
				endcase			
			end
		end
	endtask: run_phase
	
endclass: sha1tile_monitor_output
