`include "sha1_if.sv"
`include "router_if.sv"
`include "sha1tile.v"
`include "sha1tile_if.sv"

import uvm_pkg::*;
`include "uvm_macros.svh"
import sha1tiletest_pkg::*;

module sha1tile_tb_top;
	import uvm_pkg::*;

	logic clk;

	//Interfaces declaration
	sha1_if 	sha1_vif();
	router_if 	router_vif();
	sha1tile_if 	sha1tile_vif();

	//Connects the Interface to the DUT
	sha1tile dut (	.clk (clk),
            		.rst (sha1tile_vif.sig_rst),
		        .routerE_req_in (sha1tile_vif.sig_req_in),
             		.routerE_data_in (sha1tile_vif.sig_data_in),
             		.routerE_rdy_in (sha1tile_vif.sig_rdy_in),
             		.routerE_req_out (sha1tile_vif.sig_req_out),
             		.routerE_data_out (sha1tile_vif.sig_data_out),
	             	.routerE_rdy_out (sha1tile_vif.sig_rdy_out),
			// Router Monitor Interface	             	
			.router_mon_req_in (router_vif.sig_req_in),
             		.router_mon_data_in (router_vif.sig_data_in),
             		.router_mon_rdy_in (router_vif.sig_rdy_in),
             		.router_mon_req_out (router_vif.sig_req_out),
             		.router_mon_data_out (router_vif.sig_data_out),
             		.router_mon_rdy_out (router_vif.sig_rdy_out),
			// SHA1 Monitor Interface
             		.SHA1_mon_req_in (sha1_vif.sig_req_in),
             		.SHA1_mon_in_addr (sha1_vif.sig_in_addr),
             		.SHA1_mon_in_write (sha1_vif.sig_in_write),
             		.SHA1_mon_in_data (sha1_vif.sig_in_data),
            		.SHA1_mon_in_reply (sha1_vif.sig_in_reply),
             		.SHA1_mon_req_out (sha1_vif.sig_req_out),
             		.SHA1_mon_out_data (sha1_vif.sig_out_data),
             		.SHA1_mon_irq (sha1_vif.sig_irq)
		);

   	initial begin
		//Registers the Interface in the configuration block so that other
		//blocks can use it
		//uvm_resource_db#(virtual sha1_if)::set
			//(.scope("ifs"), .name("SHA1_vif"), .val(sha1_vif));
		//uvm_resource_db#(virtual router_if)::set
			//(.scope("ifs"), .name("ROUTER_vif"), .val(router_vif));
		//uvm_resource_db#(virtual sha1tile_if)::set
			//(.scope("ifs"), .name("SHA1TILE_vif"), .val(sha1tile_vif));

		uvm_config_db #(virtual sha1_if)::set(null,"uvm_test_top","SHA1_vif" , sha1_vif);
		uvm_config_db #(virtual router_if)::set(null,"uvm_test_top","ROUTER_vif" , router_vif);
		uvm_config_db #(virtual sha1tile_if)::set(null,"uvm_test_top","SHA1TILE_vif" , sha1tile_vif);

		//Executes the test
		run_test();
	end

	//Clock initialization
	initial begin
		clk = 1'b1;
	end

	//Clock generation
	always
		#5 clk = ~clk;

	// Assignments
	assign sha1tile_vif.sig_clk = clk;
	assign router_vif.sig_clk = clk;
	assign sha1_vif.sig_clk = clk;
	assign router_vif.sig_rst = 0;
	assign sha1_vif.sig_rst = 0;
	assign sha1_vif.sig_in_burst = 0;
	assign sha1_vif.sig_in_mask = 0;
	assign sha1_vif.sig_in_exclusive = 0;
	assign sha1_vif.sig_in_sender = 0;
	assign sha1_vif.sig_rdy_out = 0;
	assign sha1_vif.sig_out_addr = 0;
	assign sha1_vif.sig_out_burst = 0;	
	assign sha1_vif.sig_out_reply = 0;
	assign sha1_vif.sig_out_write = 0;
	assign sha1_vif.sig_out_mask = 0;
	assign sha1_vif.sig_out_exclusive = 0;
	assign sha1_vif.sig_out_sender = 0;
	assign sha1_vif.sig_rdy_in = 0;
	
endmodule
