
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

use work.types.all;
use work.components.all;

-------------------------------------------------------------------------------
-- Toplevel
-------------------------------------------------------------------------------

entity shmac_toplevel is
  port
  (
    PCLK             : in  std_logic;
    CLK0             : in  std_logic;
    nRESET           : in  std_logic;
    PADDR            : in  std_logic_vector(7 downto 0);
    PWRITE           : in  std_logic;
    PSEL             : in  std_logic;
    PENABLE          : in  std_logic;
    PWDATA           : in  std_logic_vector(31 downto 0);
    PRDATA           : out std_logic_vector(31 downto 0);
    PREADY           : out std_logic;
    PSLVERR          : out std_logic;

    RAM_A_SnWBYTE    : out std_logic_vector( 7 downto 0);   
    RAM_A_SnOE       : out std_logic;
    RAM_A_SnCE       : out std_logic;
    RAM_A_SADVnLD    : out std_logic;
    RAM_A_SnWR       : out std_logic;
    RAM_A_SnCKE      : out std_logic;
    RAM_A_SMODE      : out std_logic;
    RAM_A_SA         : out std_logic_vector(23 downto 3);        
    RAM_A_SD         : inout std_logic_vector(63 downto 0);

    RAM_B_SnWBYTE    : out std_logic_vector( 7 downto 0);   
    RAM_B_SnOE       : out std_logic;
    RAM_B_SnCE       : out std_logic;
    RAM_B_SADVnLD    : out std_logic;
    RAM_B_SnWR       : out std_logic;
    RAM_B_SnCKE      : out std_logic;
    RAM_B_SMODE      : out std_logic;
    RAM_B_SA         : out std_logic_vector(23 downto 3);        
    RAM_B_SD         : inout std_logic_vector(63 downto 0);
    INT              : out std_logic
  );

end shmac_toplevel;

architecture Behavioral of shmac_toplevel is
  
  constant n_tiles : tile_coord := ( i => 2, j => 3 );

  -- Signal for connection of all tiles
  constant n_total_tiles : natural := n_tiles.i * n_tiles.j;
  type packet_double_vector  is array (n_total_tiles-1 downto 0) of packet_vector;
  type boolean_double_vector is array (n_total_tiles-1 downto 0) of direction_boolean_vector;
  signal rx_req_vector    : boolean_double_vector := (others => (others => false));
  signal rx_packet_vector : packet_double_vector;
  signal rx_ack_vector    : boolean_double_vector := (others => (others => false)); 
  signal tx_req_vector    : boolean_double_vector := (others => (others => false));
  signal tx_packet_vector : packet_double_vector;
  signal tx_ack_vector    : boolean_double_vector := (others => (others => false));

  signal reset_cpu : std_logic;
  signal reset_ic : std_logic;

  signal shmac_irq : std_logic;
  signal zero : std_logic := '0';
  signal one : std_logic := '1';

  signal system_rdy : std_logic;

  signal set_debugger_mode : std_logic_vector(7 downto 0);
  signal debugger_mode : std_logic;
  signal breakpoint : std_logic_vector(7 downto 0);
  signal step : std_logic_vector(7 downto 0);
  signal reg_write : std_logic_vector(7 downto 0);
  signal reg_addr : std_logic_vector(4 downto 0);
  signal reg_write_value : std_logic_vector(31 downto 0);
  signal reg_read_value0 : std_logic_vector(31 downto 0);
  signal reg_read_value1 : std_logic_vector(31 downto 0);
  signal reg_read_value2 : std_logic_vector(31 downto 0);
  signal reg_read_value3 : std_logic_vector(31 downto 0);
  signal reg_read_value4 : std_logic_vector(31 downto 0);
  signal reg_read_value5 : std_logic_vector(31 downto 0);
  signal reg_read_value6 : std_logic_vector(31 downto 0);
  signal reg_read_value7 : std_logic_vector(31 downto 0);
  signal pc_write : std_logic_vector(7 downto 0);
  signal pc_write_value : std_logic_vector(31 downto 0);
  signal pc_read_value0 : std_logic_vector(31 downto 0);
  signal pc_read_value1 : std_logic_vector(31 downto 0);
  signal pc_read_value2 : std_logic_vector(31 downto 0);
  signal pc_read_value3 : std_logic_vector(31 downto 0);
  signal pc_read_value4 : std_logic_vector(31 downto 0);
  signal pc_read_value5 : std_logic_vector(31 downto 0);
  signal pc_read_value6 : std_logic_vector(31 downto 0);
  signal pc_read_value7 : std_logic_vector(31 downto 0);
  signal csr_write : std_logic_vector(7 downto 0);
  signal csr_addr : std_logic_vector(4 downto 0);
  signal csr_write_value : std_logic_vector(31 downto 0);
  signal csr_read_value0 : std_logic_vector(31 downto 0);
  signal csr_read_value1 : std_logic_vector(31 downto 0);
  signal csr_read_value2 : std_logic_vector(31 downto 0);
  signal csr_read_value3 : std_logic_vector(31 downto 0);
  signal csr_read_value4 : std_logic_vector(31 downto 0);
  signal csr_read_value5 : std_logic_vector(31 downto 0);
  signal csr_read_value6 : std_logic_vector(31 downto 0);
  signal csr_read_value7 : std_logic_vector(31 downto 0);
  
begin 
    apb_tile_u: apb_tile
    generic map
    (
      coord => (i => 0, j => 0),
      apb_i => 0,
      apb_j => 0,
      zbt_i => 0,
      zbt_j => 1,
      acc0_i => 1,
      acc0_j => 1,
      acc1_i => 0,
      acc1_j => 0,
      acc2_i => 0,
      acc2_j => 0,
      acc3_i => 0,
      acc3_j => 0,
      acc4_i => 0,
      acc4_j => 0,
      acc5_i => 0,
      acc5_j => 0,
      acc6_i => 0,
      acc6_j => 0,
      acc7_i => 0,
      acc7_j => 0,
      bram0_i => 0,
      bram0_j => 2,
      bram1_i => 1,
      bram1_j => 2,
      bram2_i => 0,
      bram2_j => 0,
      bram3_i => 0,
      bram3_j => 0,
      bram4_i => 0,
      bram4_j => 0,
      bram5_i => 0,
      bram5_j => 0,
      bram6_i => 0,
      bram6_j => 0,
      bram7_i => 0,
      bram7_j => 0,
      bram8_i => 0,
      bram8_j => 0,
      bram9_i => 0,
      bram9_j => 0,
      brama_i => 0,
      brama_j => 0,
      bramb_i => 0,
      bramb_j => 0,
      bramc_i => 0,
      bramc_j => 0,
      bramd_i => 0,
      bramd_j => 0,
      brame_i => 0,
      brame_j => 0,
      bramf_i => 0,
      bramf_j => 0,
      cpu_count => 1
    )
    port map
    (
      clk    => CLK0,

      reset_cpu => reset_cpu,
      reset_ic => reset_ic,

      shmac_irq => shmac_irq,
      host_irq => INT,

      system_rdy => system_rdy,

      set_debugger_mode => set_debugger_mode,
      debugger_mode => debugger_mode,
      breakpoint => breakpoint,
      step => step,
      reg_write => reg_write,
      reg_addr => reg_addr,
      reg_write_value => reg_write_value,
      reg_read_value0 => reg_read_value0,
      reg_read_value1 => reg_read_value1,
      reg_read_value2 => reg_read_value2,
      reg_read_value3 => reg_read_value3,
      reg_read_value4 => reg_read_value4,
      reg_read_value5 => reg_read_value5,
      reg_read_value6 => reg_read_value6,
      reg_read_value7 => reg_read_value7,
      pc_write => pc_write,
      pc_write_value => pc_write_value,
      pc_read_value0 => pc_read_value0,
      pc_read_value1 => pc_read_value1,
      pc_read_value2 => pc_read_value2,
      pc_read_value3 => pc_read_value3,
      pc_read_value4 => pc_read_value4,
      pc_read_value5 => pc_read_value5,
      pc_read_value6 => pc_read_value6,
      pc_read_value7 => pc_read_value7,
      csr_write => csr_write,
      csr_addr => csr_addr,
      csr_write_value => csr_write_value,
      csr_read_value0 => csr_read_value0,
      csr_read_value1 => csr_read_value1,
      csr_read_value2 => csr_read_value2,
      csr_read_value3 => csr_read_value3,
      csr_read_value4 => csr_read_value4,
      csr_read_value5 => csr_read_value5,
      csr_read_value6 => csr_read_value6,
      csr_read_value7 => csr_read_value7,

      rx_req_vector    => rx_req_vector    (0*n_tiles.j+0),
      rx_packet_vector => rx_packet_vector (0*n_tiles.j+0),
      rx_ack_vector    => rx_ack_vector    (0*n_tiles.j+0),
      tx_req_vector    => tx_req_vector    (0*n_tiles.j+0),
      tx_packet_vector => tx_packet_vector (0*n_tiles.j+0),
      tx_ack_vector    => tx_ack_vector    (0*n_tiles.j+0),

      PCLK    => PCLK,
      nRESET  => nRESET,
      PADDR   => PADDR,
      PWRITE  => PWRITE,
      PSEL    => PSEL,
      PENABLE => PENABLE,
      PWDATA  => PWDATA,
      PRDATA  => PRDATA,
      PREADY  => PREADY,
      PSLVERR => PSLVERR
    );

    ramtile_u: zbtram_tile
    generic map
    (
      coord => (i => 0, j => 1),
      apb_i => 0,
      apb_j => 0,
      zbt_i => 0,
      zbt_j => 1,
      acc0_i => 1,
      acc0_j => 1,
      acc1_i => 0,
      acc1_j => 0,
      acc2_i => 0,
      acc2_j => 0,
      acc3_i => 0,
      acc3_j => 0,
      acc4_i => 0,
      acc4_j => 0,
      acc5_i => 0,
      acc5_j => 0,
      acc6_i => 0,
      acc6_j => 0,
      acc7_i => 0,
      acc7_j => 0,
      bram0_i => 0,
      bram0_j => 2,
      bram1_i => 1,
      bram1_j => 2,
      bram2_i => 0,
      bram2_j => 0,
      bram3_i => 0,
      bram3_j => 0,
      bram4_i => 0,
      bram4_j => 0,
      bram5_i => 0,
      bram5_j => 0,
      bram6_i => 0,
      bram6_j => 0,
      bram7_i => 0,
      bram7_j => 0,
      bram8_i => 0,
      bram8_j => 0,
      bram9_i => 0,
      bram9_j => 0,
      brama_i => 0,
      brama_j => 0,
      bramb_i => 0,
      bramb_j => 0,
      bramc_i => 0,
      bramc_j => 0,
      bramd_i => 0,
      bramd_j => 0,
      brame_i => 0,
      brame_j => 0,
      bramf_i => 0,
      bramf_j => 0
    )
    port map
    (
      clk    => CLK0,
      reset_ic => reset_ic,

      rx_req_vector    => rx_req_vector    (0*n_tiles.j+1),
      rx_packet_vector => rx_packet_vector (0*n_tiles.j+1),
      rx_ack_vector    => rx_ack_vector    (0*n_tiles.j+1),
      tx_req_vector    => tx_req_vector    (0*n_tiles.j+1),
      tx_packet_vector => tx_packet_vector (0*n_tiles.j+1),
      tx_ack_vector    => tx_ack_vector    (0*n_tiles.j+1),

      RAM_A_SnWBYTE => RAM_A_SnWBYTE,
      RAM_A_SnOE => RAM_A_SnOE,
      RAM_A_SnCE => RAM_A_SnCE,
      RAM_A_SADVnLD => RAM_A_SADVnLD,
      RAM_A_SnWR => RAM_A_SnWR,
      RAM_A_SnCKE => RAM_A_SnCKE,
      RAM_A_SMODE => RAM_A_SMODE,
      RAM_A_SA => RAM_A_SA,
      RAM_A_SD => RAM_A_SD,

      RAM_B_SnWBYTE => RAM_B_SnWBYTE,
      RAM_B_SnOE => RAM_B_SnOE,
      RAM_B_SnCE => RAM_B_SnCE,
      RAM_B_SADVnLD => RAM_B_SADVnLD,
      RAM_B_SnWR => RAM_B_SnWR,
      RAM_B_SnCKE => RAM_B_SnCKE,
      RAM_B_SMODE => RAM_B_SMODE,
      RAM_B_SA => RAM_B_SA,
      RAM_B_SD => RAM_B_SD
    );

    ram_0_2_u: ram_tile
    generic map
    (
      coord => (i => 0, j => 2),
      apb_i => 0,
      apb_j => 0,
      zbt_i => 0,
      zbt_j => 1,
      acc0_i => 1,
      acc0_j => 1,
      acc1_i => 0,
      acc1_j => 0,
      acc2_i => 0,
      acc2_j => 0,
      acc3_i => 0,
      acc3_j => 0,
      acc4_i => 0,
      acc4_j => 0,
      acc5_i => 0,
      acc5_j => 0,
      acc6_i => 0,
      acc6_j => 0,
      acc7_i => 0,
      acc7_j => 0,
      bram0_i => 0,
      bram0_j => 2,
      bram1_i => 1,
      bram1_j => 2,
      bram2_i => 0,
      bram2_j => 0,
      bram3_i => 0,
      bram3_j => 0,
      bram4_i => 0,
      bram4_j => 0,
      bram5_i => 0,
      bram5_j => 0,
      bram6_i => 0,
      bram6_j => 0,
      bram7_i => 0,
      bram7_j => 0,
      bram8_i => 0,
      bram8_j => 0,
      bram9_i => 0,
      bram9_j => 0,
      brama_i => 0,
      brama_j => 0,
      bramb_i => 0,
      bramb_j => 0,
      bramc_i => 0,
      bramc_j => 0,
      bramd_i => 0,
      bramd_j => 0,
      brame_i => 0,
      brame_j => 0,
      bramf_i => 0,
      bramf_j => 0
    )
    port map
    (
      clk              => CLK0,
      reset_ic         => reset_ic,
      rx_req_vector    => rx_req_vector    (0*n_tiles.j+2),
      rx_packet_vector => rx_packet_vector (0*n_tiles.j+2),
      rx_ack_vector    => rx_ack_vector    (0*n_tiles.j+2),
      tx_req_vector    => tx_req_vector    (0*n_tiles.j+2),
      tx_packet_vector => tx_packet_vector (0*n_tiles.j+2),
      tx_ack_vector    => tx_ack_vector    (0*n_tiles.j+2)
    );

    sodor_0_u: sodor_tile
    generic map
    (
      coord => (i => 1, j => 0),
      apb_i => 0,
      apb_j => 0,
      zbt_i => 0,
      zbt_j => 1,
      acc0_i => 1,
      acc0_j => 1,
      acc1_i => 0,
      acc1_j => 0,
      acc2_i => 0,
      acc2_j => 0,
      acc3_i => 0,
      acc3_j => 0,
      acc4_i => 0,
      acc4_j => 0,
      acc5_i => 0,
      acc5_j => 0,
      acc6_i => 0,
      acc6_j => 0,
      acc7_i => 0,
      acc7_j => 0,
      bram0_i => 0,
      bram0_j => 2,
      bram1_i => 1,
      bram1_j => 2,
      bram2_i => 0,
      bram2_j => 0,
      bram3_i => 0,
      bram3_j => 0,
      bram4_i => 0,
      bram4_j => 0,
      bram5_i => 0,
      bram5_j => 0,
      bram6_i => 0,
      bram6_j => 0,
      bram7_i => 0,
      bram7_j => 0,
      bram8_i => 0,
      bram8_j => 0,
      bram9_i => 0,
      bram9_j => 0,
      brama_i => 0,
      brama_j => 0,
      bramb_i => 0,
      bramb_j => 0,
      bramc_i => 0,
      bramc_j => 0,
      bramd_i => 0,
      bramd_j => 0,
      brame_i => 0,
      brame_j => 0,
      bramf_i => 0,
      bramf_j => 0,
      cpu_id => 0
    )
    port map
    (
      clk              => CLK0,
      reset_cpu        => reset_cpu,
      reset_ic         => reset_ic,
      irq              => shmac_irq,
      system_rdy       => one,

      set_debugger_mode => set_debugger_mode(0),
      debugger_mode => debugger_mode,
      breakpoint => breakpoint(0),
      step => step(0),
      reg_write => reg_write(0),
      reg_addr => reg_addr,
      reg_write_value => reg_write_value,
      reg_read_value => reg_read_value0,
      pc_write => pc_write(0),
      pc_write_value => pc_write_value,
      pc_read_value => pc_read_value0,
      csr_write => csr_write(0),
      csr_addr => csr_addr,
      csr_write_value => csr_write_value,
      csr_read_value => csr_read_value0,
      
      rx_req_vector    => rx_req_vector    (1*n_tiles.j+0),
      rx_packet_vector => rx_packet_vector (1*n_tiles.j+0),
      rx_ack_vector    => rx_ack_vector    (1*n_tiles.j+0),
      tx_req_vector    => tx_req_vector    (1*n_tiles.j+0),
      tx_packet_vector => tx_packet_vector (1*n_tiles.j+0),
      tx_ack_vector    => tx_ack_vector    (1*n_tiles.j+0)
    );

    sha1_1_1_u: sha1_tile
    generic map
    (
      coord => (i => 1, j => 1),
      apb_i => 0,
      apb_j => 0,
      zbt_i => 0,
      zbt_j => 1,
      acc0_i => 1,
      acc0_j => 1,
      acc1_i => 0,
      acc1_j => 0,
      acc2_i => 0,
      acc2_j => 0,
      acc3_i => 0,
      acc3_j => 0,
      acc4_i => 0,
      acc4_j => 0,
      acc5_i => 0,
      acc5_j => 0,
      acc6_i => 0,
      acc6_j => 0,
      acc7_i => 0,
      acc7_j => 0,
      bram0_i => 0,
      bram0_j => 2,
      bram1_i => 1,
      bram1_j => 2,
      bram2_i => 0,
      bram2_j => 0,
      bram3_i => 0,
      bram3_j => 0,
      bram4_i => 0,
      bram4_j => 0,
      bram5_i => 0,
      bram5_j => 0,
      bram6_i => 0,
      bram6_j => 0,
      bram7_i => 0,
      bram7_j => 0,
      bram8_i => 0,
      bram8_j => 0,
      bram9_i => 0,
      bram9_j => 0,
      brama_i => 0,
      brama_j => 0,
      bramb_i => 0,
      bramb_j => 0,
      bramc_i => 0,
      bramc_j => 0,
      bramd_i => 0,
      bramd_j => 0,
      brame_i => 0,
      brame_j => 0,
      bramf_i => 0,
      bramf_j => 0
    )
    port map
    (
      clk              => CLK0,
      reset_cpu        => reset_cpu,
      reset_ic         => reset_ic,
      irq              => open,

      rx_req_vector    => rx_req_vector    (1*n_tiles.j+1),
      rx_packet_vector => rx_packet_vector (1*n_tiles.j+1),
      rx_ack_vector    => rx_ack_vector    (1*n_tiles.j+1),
      tx_req_vector    => tx_req_vector    (1*n_tiles.j+1),
      tx_packet_vector => tx_packet_vector (1*n_tiles.j+1),
      tx_ack_vector    => tx_ack_vector    (1*n_tiles.j+1)
    );

    ram_1_2_u: ram_tile
    generic map
    (
      coord => (i => 1, j => 2),
      apb_i => 0,
      apb_j => 0,
      zbt_i => 0,
      zbt_j => 1,
      acc0_i => 1,
      acc0_j => 1,
      acc1_i => 0,
      acc1_j => 0,
      acc2_i => 0,
      acc2_j => 0,
      acc3_i => 0,
      acc3_j => 0,
      acc4_i => 0,
      acc4_j => 0,
      acc5_i => 0,
      acc5_j => 0,
      acc6_i => 0,
      acc6_j => 0,
      acc7_i => 0,
      acc7_j => 0,
      bram0_i => 0,
      bram0_j => 2,
      bram1_i => 1,
      bram1_j => 2,
      bram2_i => 0,
      bram2_j => 0,
      bram3_i => 0,
      bram3_j => 0,
      bram4_i => 0,
      bram4_j => 0,
      bram5_i => 0,
      bram5_j => 0,
      bram6_i => 0,
      bram6_j => 0,
      bram7_i => 0,
      bram7_j => 0,
      bram8_i => 0,
      bram8_j => 0,
      bram9_i => 0,
      bram9_j => 0,
      brama_i => 0,
      brama_j => 0,
      bramb_i => 0,
      bramb_j => 0,
      bramc_i => 0,
      bramc_j => 0,
      bramd_i => 0,
      bramd_j => 0,
      brame_i => 0,
      brame_j => 0,
      bramf_i => 0,
      bramf_j => 0
    )
    port map
    (
      clk              => CLK0,
      reset_ic         => reset_ic,
      rx_req_vector    => rx_req_vector    (1*n_tiles.j+2),
      rx_packet_vector => rx_packet_vector (1*n_tiles.j+2),
      rx_ack_vector    => rx_ack_vector    (1*n_tiles.j+2),
      tx_req_vector    => tx_req_vector    (1*n_tiles.j+2),
      tx_packet_vector => tx_packet_vector (1*n_tiles.j+2),
      tx_ack_vector    => tx_ack_vector    (1*n_tiles.j+2)
    );

    breakpoint(1) <= '0';
    reg_read_value1 <= (others => '0');
    pc_read_value1 <= (others => '0');
    csr_read_value1 <= (others => '0');

    breakpoint(2) <= '0';
    reg_read_value2 <= (others => '0');
    pc_read_value2 <= (others => '0');
    csr_read_value2 <= (others => '0');

    breakpoint(3) <= '0';
    reg_read_value3 <= (others => '0');
    pc_read_value3 <= (others => '0');
    csr_read_value3 <= (others => '0');

    breakpoint(4) <= '0';
    reg_read_value4 <= (others => '0');
    pc_read_value4 <= (others => '0');
    csr_read_value4 <= (others => '0');

    breakpoint(5) <= '0';
    reg_read_value5 <= (others => '0');
    pc_read_value5 <= (others => '0');
    csr_read_value5 <= (others => '0');

    breakpoint(6) <= '0';
    reg_read_value6 <= (others => '0');
    pc_read_value6 <= (others => '0');
    csr_read_value6 <= (others => '0');

    breakpoint(7) <= '0';
    reg_read_value7 <= (others => '0');
    pc_read_value7 <= (others => '0');
    csr_read_value7 <= (others => '0');

  -------------------------------------------------------------------------------
  -- Connect all tiles
  -------------------------------------------------------------------------------
  -- Vertical
  connect_v_i: for i in 1 to n_tiles.i-1 generate 
    connect_v_j: for j in 0 to n_tiles.j-1 generate
      -- N -> S
      rx_req_vector   (     i*n_tiles.j + j     )(north) <= tx_req_vector   ( (i-1)*n_tiles.j + j     )(south);
      rx_packet_vector(     i*n_tiles.j + j     )(north) <= tx_packet_vector( (i-1)*n_tiles.j + j     )(south);
      tx_ack_vector   ( (i-1)*n_tiles.j + j     )(south) <= rx_ack_vector   (     i*n_tiles.j + j     )(north);
      -- N <- S
      rx_req_vector   ( (i-1)*n_tiles.j + j     )(south) <= tx_req_vector   (     i*n_tiles.j + j     )(north);
      rx_packet_vector( (i-1)*n_tiles.j + j     )(south) <= tx_packet_vector(     i*n_tiles.j + j     )(north);
      tx_ack_vector   (     i*n_tiles.j + j     )(north) <= rx_ack_vector   ( (i-1)*n_tiles.j + j     )(south);
    end generate;
  end generate;

  -- Torus
  connect_vt_j: for j in 0 to n_tiles.j-1 generate
    -- N -> S
    rx_req_vector   (                           j )(north) <= tx_req_vector   ( (n_tiles.i-1)*n_tiles.j + j )(south);
    rx_packet_vector(                           j )(north) <= tx_packet_vector( (n_tiles.i-1)*n_tiles.j + j )(south);
    tx_ack_vector   ( (n_tiles.i-1)*n_tiles.j + j )(south) <= rx_ack_vector   (                           j )(north);
    -- N <- S
    rx_req_vector   ( (n_tiles.i-1)*n_tiles.j + j )(south) <= tx_req_vector   (                           j )(north);
    rx_packet_vector( (n_tiles.i-1)*n_tiles.j + j )(south) <= tx_packet_vector(                           j )(north);
    tx_ack_vector   (                           j )(north) <= rx_ack_vector   ( (n_tiles.i-1)*n_tiles.j + j )(south);
  end generate;

  -- Horizontal
  connect_h_i: for i in 0 to n_tiles.i-1 generate 
    connect_h_j: for j in 1 to n_tiles.j-1 generate
      -- W -> E
      rx_req_vector   (     i*n_tiles.j + j     )(west) <= tx_req_vector   (      i*n_tiles.j + (j-1) )(east);
      rx_packet_vector(     i*n_tiles.j + j     )(west) <= tx_packet_vector(      i*n_tiles.j + (j-1) )(east);
      tx_ack_vector   (     i*n_tiles.j + (j-1) )(east) <= rx_ack_vector   (      i*n_tiles.j + j     )(west);
      -- W <- E
      rx_req_vector   (     i*n_tiles.j + (j-1) )(east) <= tx_req_vector   (      i*n_tiles.j + j     )(west);
      rx_packet_vector(     i*n_tiles.j + (j-1) )(east) <= tx_packet_vector(      i*n_tiles.j + j     )(west);
      tx_ack_vector   (     i*n_tiles.j + j     )(west) <= rx_ack_vector   (      i*n_tiles.j + (j-1) )(east);
    end generate;
  end generate;

  -- Torus
  connect_ht_i: for i in 0 to n_tiles.i-1 generate
    -- W -> E
    rx_req_vector   ( i*n_tiles.j + 0           )(west) <= tx_req_vector   ( i*n_tiles.j + n_tiles.j-1 )(east);
    rx_packet_vector( i*n_tiles.j + 0           )(west) <= tx_packet_vector( i*n_tiles.j + n_tiles.j-1 )(east);
    tx_ack_vector   ( i*n_tiles.j + n_tiles.j-1 )(east) <= rx_ack_vector   ( i*n_tiles.j + 0           )(west);
    -- W <- E
    rx_req_vector   ( i*n_tiles.j + n_tiles.j-1 )(east) <= tx_req_vector   ( i*n_tiles.j + 0           )(west);
    rx_packet_vector( i*n_tiles.j + n_tiles.j-1 )(east) <= tx_packet_vector( i*n_tiles.j + 0           )(west);
    tx_ack_vector   ( i*n_tiles.j + 0           )(west) <= rx_ack_vector   ( i*n_tiles.j + n_tiles.j-1 )(east);
  end generate;

end Behavioral;
