//-----------------------------------------------------------------------------
// Title         : SHMAC toplevel testbench
// Project       : SHMAC
//-----------------------------------------------------------------------------
// File          : shmac_toplevel_tb.v
// Author        : Edward Mitacc  
// Created       : 24.05.2016
// Last modified : //-----------------------------------------------------------------------------
// Description :
// 
//-----------------------------------------------------------------------------
// Copyright (c) 2016 by ARM/CARD
//------------------------------------------------------------------------------
// Modification history :
// ??.08.2013 : created
//-----------------------------------------------------------------------------

`timescale 1fs / 1fs

`include "common_defs.v"

module shmac_toplevel_tb;
   
   `define SHA1FILE "sha1/sha1.hex"	
   `define HEXFILE "toplevel/shmac_test_riscv/shmac.hex"

   `define HOSTREG_ADDR 8'h0
   `define HOSTREG_DATA 8'h1
   `define HOSTREG_RESET 8'h2
   `define HOSTREG_SHMAC_INT_STATUS 8'h3
   `define HOSTREG_HOST_INT_STATUS 8'h4

   `define HOSTREG_SYS_OUT_DATA 8'h10
   `define HOSTREG_SYS_IN_DATA 8'h20
   
   `define PHASEDONE 32'had

   // APB interface
   reg                      PCLK;
   reg                      CLK0;
   reg                      nRESET;
   reg [7:0]                PADDR;
   reg                      PWRITE;
   reg                      PSEL;
   reg                      PENABLE;
   reg [31:0]               PWDATA;
   wire [31:0]              PRDATA;
   wire                     PREADY;
   wire                     PSLVERR;
   wire                     INT;

   // ZBT interface
   wire [7:0]               RAM_A_SnWBYTE;
   wire                     RAM_A_SnOE;
   wire                     RAM_A_SnCE;
   wire                     RAM_A_SADVnLD;
   wire                     RAM_A_SnWR;
   wire                     RAM_A_SnCKE;
   wire                     RAM_A_SMODE;
   wire [22:2]              RAM_A_SA;
   wire [63:0]              RAM_A_SD;
   wire [7:0]               RAM_B_SnWBYTE;
   wire                     RAM_B_SnOE;
   wire                     RAM_B_SnCE;
   wire                     RAM_B_SADVnLD;
   wire                     RAM_B_SnWR;
   wire                     RAM_B_SnCKE;
   wire                     RAM_B_SMODE;
   wire [22:2]              RAM_B_SA;
   wire [63:0]              RAM_B_SD;

   integer                  file, topfile, topfile_size, topfile_binary, topfile_mem_loc;
   integer                  n, i, x, y, loop, loop2;

   reg [31:0]               instr, data, channel;

   `define RAM_SIZE 8388608

   reg [63:0] ram_a [`RAM_SIZE:0];
   reg [63:0] ram_b [`RAM_SIZE:0];

   reg [63:0] RAM_A_data;
   reg [63:0] RAM_B_data;

   reg        write_a, write_b;
   reg [31:0] binary [0:131072];

   /////////////////////////////////////////////////////////////////////////////

   shmac_toplevel dut
     (
      .PCLK (PCLK),
      .CLK0 (PCLK),
      .nRESET (nRESET),
      .PADDR (PADDR),
      .PWRITE (PWRITE),
      .PSEL (PSEL),
      .PENABLE (PENABLE),
      .PWDATA (PWDATA),
      .PRDATA (PRDATA),
      .PREADY (PREADY),
      .PSLVERR (PSLVERR),
      .INT (INT),
      
      .RAM_A_SnWBYTE (RAM_A_SnWBYTE),
      .RAM_A_SnOE (RAM_A_SnOE),
      .RAM_A_SnCE (RAM_A_SnCE),
      .RAM_A_SADVnLD (RAM_A_SADVnLD),
      .RAM_A_SnWR (RAM_A_SnWR),
      .RAM_A_SnCKE (RAM_A_SnCKE),
      .RAM_A_SMODE (RAM_A_SMODE),
      .RAM_A_SA (RAM_A_SA),
      .RAM_A_SD (RAM_A_SD),

      .RAM_B_SnWBYTE (RAM_B_SnWBYTE),
      .RAM_B_SnOE (RAM_B_SnOE),
      .RAM_B_SnCE (RAM_B_SnCE),
      .RAM_B_SADVnLD (RAM_B_SADVnLD),
      .RAM_B_SnWR (RAM_B_SnWR),
      .RAM_B_SnCKE (RAM_B_SnCKE),
      .RAM_B_SMODE (RAM_B_SMODE),
      .RAM_B_SA (RAM_B_SA),
      .RAM_B_SD (RAM_B_SD)
      );

   /////////////////////////////////////////////////////////////////////////////

   task apb_write;
      input [31:0] addr;
      input [31:0] data;

      begin
        @(posedge PCLK);
        PADDR = addr;
        PWRITE = 1'b1;
        PSEL = 1'b1;
        PWDATA = data;

        @(posedge PCLK);
        PENABLE = 1'b1;
        
        @(posedge PCLK);
        
        while(!PREADY) begin
           @(posedge PCLK);
        end

        PENABLE = 1'b0;
        PSEL = 1'b0;
      end
   endtask

   task apb_read;
      input [31:0] addr;
      output [31:0] data;

      begin
        @(posedge PCLK);
        PADDR = addr;
        PWRITE = 1'b0;
        PSEL = 1'b1;

        @(posedge PCLK);
        PENABLE = 1'b1;
        
        @(posedge PCLK);
        
        while(!PREADY) begin
           @(posedge PCLK);
        end

        data = PRDATA;

        PENABLE = 1'b0;
        PSEL = 1'b0;
      end
   endtask
   /////////////////////////////////////////////////////////////////////////////

   initial 
     begin 
        PCLK = 0; 
        CLK0 = 0; 

        nRESET = 0;

        PADDR = 0;
        PWRITE = 0;
        PSEL = 0;
        PENABLE = 0;
        PWDATA = 0;
     end 
   
   always 
     #8000000 CLK0 = !CLK0; 

   always
     #8000000 PCLK = !PCLK;

   /////////////////////////////////////////////////////////////////////////////
   // ZBT RAM simulator

   assign RAM_A_SD = RAM_A_data;

   initial
     begin
        for(x = 0; x < `RAM_SIZE; x = x + 1) begin
           ram_a[x] = 64'h0; //Undefined instructions
        end

        while(1) begin
           RAM_A_data = 64'hZZZZZZZZZZZZZZZZ;

           // wait until access
           while(RAM_A_SnCE) @(posedge PCLK);

           if(!RAM_A_SnWR) begin
              if(!RAM_A_SnWBYTE[0]) begin
                 ram_a[RAM_A_SA][7:0] = RAM_A_SD[7:0];
              end
              if(!RAM_A_SnWBYTE[1]) begin
                 ram_a[RAM_A_SA][15:8] = RAM_A_SD[15:8];
              end
              if(!RAM_A_SnWBYTE[2]) begin
                 ram_a[RAM_A_SA][23:16] = RAM_A_SD[23:16];
              end
              if(!RAM_A_SnWBYTE[3]) begin
                 ram_a[RAM_A_SA][31:24] = RAM_A_SD[31:24];
              end
              if(!RAM_A_SnWBYTE[4]) begin
                 ram_a[RAM_A_SA][39:32] = RAM_A_SD[39:32];
              end
              if(!RAM_A_SnWBYTE[5]) begin
                 ram_a[RAM_A_SA][47:40] = RAM_A_SD[47:40];
              end
              if(!RAM_A_SnWBYTE[6]) begin
                 ram_a[RAM_A_SA][55:48] = RAM_A_SD[55:48];
              end
              if(!RAM_A_SnWBYTE[7]) begin
                 ram_a[RAM_A_SA][63:56] = RAM_A_SD[63:56];
              end
              @(posedge PCLK);
           end
           else begin
              @(posedge PCLK);
              RAM_A_data = ram_a[RAM_A_SA];
           end

           @(posedge PCLK);
        end
     end

   assign RAM_B_SD = RAM_B_data;

   initial
     begin
        for(y = 0; y < `RAM_SIZE; y = y + 1) begin
           ram_b[y] = 64'h0; //Undefined instructions
        end

        while(1) begin
           RAM_B_data = 64'hZZZZZZZZZZZZZZZZ;

           // wait until access
           while(RAM_B_SnCE) @(posedge PCLK);

           if(!RAM_B_SnWR) begin
              if(!RAM_B_SnWBYTE[0]) begin
                 ram_b[RAM_B_SA][7:0] = RAM_B_SD[7:0];
              end
              if(!RAM_B_SnWBYTE[1]) begin
                 ram_b[RAM_B_SA][15:8] = RAM_B_SD[15:8];
              end
              if(!RAM_B_SnWBYTE[2]) begin
                 ram_b[RAM_B_SA][23:16] = RAM_B_SD[23:16];
              end
              if(!RAM_B_SnWBYTE[3]) begin
                 ram_b[RAM_B_SA][31:24] = RAM_B_SD[31:24];
              end
              if(!RAM_B_SnWBYTE[4]) begin
                 ram_b[RAM_B_SA][39:32] = RAM_B_SD[39:32];
              end
              if(!RAM_B_SnWBYTE[5]) begin
                 ram_b[RAM_B_SA][47:40] = RAM_B_SD[47:40];
              end
              if(!RAM_B_SnWBYTE[6]) begin
                 ram_b[RAM_B_SA][55:48] = RAM_B_SD[55:48];
              end
              if(!RAM_B_SnWBYTE[7]) begin
                 ram_b[RAM_B_SA][63:56] = RAM_B_SD[63:56];
              end
              @(posedge PCLK);
           end
           else begin
              @(posedge PCLK);
              RAM_B_data = ram_b[RAM_B_SA];
           end

           @(posedge PCLK);
        end
     end

   /////////////////////////////////////////////////////////////////////////////

   initial
     begin
        // read file
        file = $fopen(`HEXFILE, "rb");
        if(file == 0) 
          begin
             $display("can't open %s", `HEXFILE);
             $stop;
          end

        @(posedge PCLK);
        @(posedge PCLK);
        @(posedge PCLK);
        @(posedge PCLK);
        @(posedge PCLK);
        @(posedge PCLK);
        nRESET = 1;

        // reset shmac
        apb_write(`HOSTREG_RESET, 3);
        @(posedge PCLK);

        // run interconnect
        apb_write(`HOSTREG_RESET, 1);
        
        // write to ram
        apb_write(`HOSTREG_ADDR, 32'h2000);
        n = 0;
        while(!$feof(file))
          begin
             $fscanf(file,"%h\n", instr);
             apb_write(`HOSTREG_DATA, instr);
             n = n + 1;
          end

        // read and verify ram
        apb_write(`HOSTREG_ADDR, 32'h2000);
        $fseek(file, 0, 0); // rewind
        for(i = 0; i < n; i = i + 1)
          begin
             $fscanf(file,"%h\n", instr);
             apb_read(`HOSTREG_DATA, data);
             if(data != instr) 
               $fatal(1, "Incorrect RAM data %x != %x\n", data, instr);
          end

        $display("[%t]: RAM written and verified\n", $time);

        // run CPU tiles
        apb_write(`HOSTREG_RESET, 0);
        
	     
	  // open file (SHA1 data)
        file = $fopen(`SHA1FILE, "rb");
        if(file == 0) begin
           $display("can't open %s", `SHA1FILE);
           $stop;
        end

        n = 0;

        // read file (SHA1 data)
        while(!$feof(file)) begin
           $fscanf(file,"%h\n", data);
           binary[n] <= 						{data[7:0],data[15:8],data[23:16],data[31:24]};
           n = n + 1;
        end

	   // write to RAM (SHA1 data)
        apb_write(`HOSTREG_ADDR, 32'h20000000);
        for(i = 0; i < n; i = i + 1)
        begin
        	data = binary[i];  
		apb_write(`HOSTREG_DATA, data);
        end
		        

     end

endmodule
